# -*- coding: utf-8 -*-
"""
Created on Tue Aug  1 09:11:11 2017
@author: david
Perform directed 'reconciliation' first and then apply EggNOG method
1 - root gene trees on outgroup: unique one this time
2 - infer orthologues
"""
import os
import sys
import csv
import glob
import argparse
import operator
import itertools
import multiprocessing as mp
from collections import defaultdict, deque

from . import tree as tree_lib
from . import resolve
try:
    import queue
except ImportError:
    import Queue as queue

PY2 = sys.version_info <= (3,)
csv_write_mode = 'wb' if PY2 else 'wt'
csv_append_mode = 'ab' if PY2 else 'at'
csv_read_mode = 'rb' if PY2 else 'rt'

debug = False   # HOGs

if not PY2:
    xrange = range


def GeneToSpecies_dash(g):
    return g.split("_", 1)[0]


OrthoFinderIDs = GeneToSpecies_dash


def GeneToSpecies_secondDash(g):
    return "_".join(g.split("_", 2)[:2])


def GeneToSpecies_3rdDash(g):
    return "_".join(g.split("_", 3)[:3])


def GeneToSpecies_dot(g):
    return g.split(".", 1)[0]


def GeneToSpecies_hyphen(g):
    return g.split("-", 1)[0]


def SpeciesAndGene_dash(g):
    return g.split("_", 1)


def SpeciesAndGene_secondDash(g):
    a, b, c = g.split("_", 2)
    return (a+"_"+b, c)


def SpeciesAndGene_3rdDash(g):
    a, b, c, d = g.split("_", 3)
    return (a+"_"+b+"_"+c, d)


def SpeciesAndGene_dot(g):
    return g.split(".", 1)


def SpeciesAndGene_hyphen(g):
    return g.split("-", 1)


SpeciesAndGene_lookup = {GeneToSpecies_dash: SpeciesAndGene_dash,
                         GeneToSpecies_secondDash: SpeciesAndGene_secondDash,
                         GeneToSpecies_3rdDash: SpeciesAndGene_3rdDash,
                         GeneToSpecies_dot: SpeciesAndGene_dot,
                         GeneToSpecies_hyphen: SpeciesAndGene_hyphen}


class RootMap(object):
    def __init__(self, setA, setB, GeneToSpecies):
        self.setA = setA
        self.setB = setB
        self.GeneToSpecies = GeneToSpecies

    def GeneMap(self, gene_name):
        sp = self.GeneToSpecies(gene_name)
        if sp in self.setA:
            return True
        elif sp in self.setB:
            return False
        else:
            print(gene_name)
            print(sp)
            raise Exception


def StoreSpeciesSets(t, GeneMap, tag="sp_"):
    tag_up = tag + "up"
    tag_down = tag + "down"
    for node in t.traverse('postorder'):
        if node.is_leaf():
            node.add_feature(tag_down, {GeneMap(node.name)})
        elif node.is_root():
            continue
        else:
            node.add_feature(tag_down, set.union(
                *[ch.__getattribute__(tag_down) for ch in node.get_children()]))
    for node in t.traverse('preorder'):
        if node.is_root():
            node.add_feature(tag_up, set())
        else:
            parent = node.up
            if parent.is_root():
                others = [ch for ch in parent.get_children() if ch != node]
                node.add_feature(tag_up, set.union(
                    *[other.__getattribute__(tag_down) for other in others]))
            else:
                others = [ch for ch in parent.get_children() if ch != node]
                sp_downs = set.union(
                    *[other.__getattribute__(tag_down) for other in others])
                node.add_feature(tag_up, parent.__getattribute__(
                    tag_up).union(sp_downs))
    t.add_feature(tag_down, set.union(
        *[ch.__getattribute__(tag_down) for ch in t.get_children()]))


"""
HOGs
-------------------------------------------------------------------------------
"""


def MRCA_node(t_rooted, taxa):
    return (t_rooted & next(taxon for taxon in taxa)) if len(taxa) == 1 else t_rooted.get_common_ancestor(taxa)


def GetHOGs_from_tree(iog, tree, hog_writer, lock_hogs, q_split_paralogous_clades):
    og_name = "OG%07d" % iog
    if debug:
        print("\n===== %s =====" % og_name)
    try:
        tree = hog_writer.mark_dups_below(tree)
        cached_hogs = []
        for n in tree.traverse("preorder"):
            cached_hogs.extend(hog_writer.write_clade_v2(
                n, og_name, q_split_paralogous_clades))
        hog_writer.WriteCachedHOGs(cached_hogs, lock_hogs)
    except:
        print("WARNING: HOG analysis for %s failed" % og_name)
        print("Please report to https://github.com/davidemms/OrthoFinder/issues including \
SpeciesTree_rooted_ids.txt and Trees_ids/%s_tree_id.txt from WorkingDirectory/" % og_name)
        print(cached_hogs)
        raise


def get_highest_nodes(nodes, comp_nodes):
    """
    Returns the nodes closest to the root
    Args:
        nodes - the list of nodes to examine
        comp_nodes - dict:NX -> ( {closer to root}, {further from root} )
    """
    return {n for n in nodes if not any(n in comp_nodes[n2][1] for n2 in nodes)}


def get_comparable_nodes(sp_tree):
    """
    Return a dictionary of comaprable nodes
    Node NX < NY if NX is on the path between NY and the root.
    If a node is not <, =, > another then they are incomparable
    Args:
        sp_tree - sp_tree with labelled nodes
    Returns:
        comp_nodes - dict:NX -> ( {n|n<NX}, {n|n>NX} ) i.e. (higher_nodes, lower_nodes)
    """
    comp_nodes = dict()
    for n in sp_tree.traverse('postorder'):
        nodes_below = set()
        if not n.is_leaf():
            for ch in n.get_children():
                if not ch.is_leaf():
                    nodes_below.update(ch.nodes_below)
                nodes_below.add(ch.name)
        above = set([nn.name for nn in n.get_ancestors()])
        n.add_feature('nodes_below', nodes_below)
        comp_nodes[n.name] = (above, nodes_below, above.union(
            nodes_below.union(set(n.name))))
    return comp_nodes


"""
Orthologs
-------------------------------------------------------------------------------
"""


def OutgroupIngroupSeparationScore(sp_up, sp_down, sett1, sett2, N_recip, n1, n2):
    f_dup = len(sp_up.intersection(sett1)) * len(sp_up.intersection(sett2)) * \
        len(sp_down.intersection(sett1)) * \
        len(sp_down.intersection(sett2)) * N_recip
    f_a = len(sp_up.intersection(sett1)) * (n2-len(sp_up.intersection(sett2))) * \
        (n1-len(sp_down.intersection(sett1))) * \
        len(sp_down.intersection(sett2)) * N_recip
    f_b = (n1-len(sp_up.intersection(sett1))) * len(sp_up.intersection(sett2)) * \
        len(sp_down.intersection(sett1)) * \
        (n2-len(sp_down.intersection(sett2))) * N_recip
    choice = (f_dup, f_a, f_b)
    return max(choice)


def GetRoots(tree, species_tree_rooted, GeneToSpecies):
    """
    Allow non-binary gene or species trees.
    (A,B,C) => consider splits A|BC, B|AC, C|AB - this applies to gene and species tree
    If a clean ingroup/outgroup split cannot be found then score root by geometric mean of fraction of expected species actually 
    observed for the two splits
    """
    speciesObserved = set([GeneToSpecies(g) for g in tree.get_leaf_names()])
    if len(speciesObserved) == 1:
        # arbitrary root if all genes are from the same species
        return [next(n for n in tree)]

    # use species tree to find correct outgroup according to what species are present in the gene tree
    n = species_tree_rooted
    children = n.get_children()
    leaves = [set(ch.get_leaf_names()) for ch in children]
    have = [len(l.intersection(speciesObserved)) != 0 for l in leaves]
    while sum(have) < 2:
        n = children[have.index(True)]
        children = n.get_children()
        leaves = [set(ch.get_leaf_names()) for ch in children]
        have = [len(l.intersection(speciesObserved)) != 0 for l in leaves]

    # Get splits to look for
    roots_list = []
    scores_list = []   # the fraction completeness of the two clades
#    roots_set = set()
    for i in xrange(len(leaves)):
        t1 = leaves[i]
        t2 = set.union(*[l for j, l in enumerate(leaves) if j != i])
        # G - set of species in gene tree
        # First relevant split in species tree is (A,B), such that A \cap G \neq \emptyset and A \cap G \neq \emptyset
        # label all nodes in gene tree according the whether subsets of A, B or both lie below node
        StoreSpeciesSets(tree, GeneToSpecies)   # sets of species
        root_mapper = RootMap(t1, t2, GeneToSpecies)
        sett1 = set(t1)
        sett2 = set(t2)
        nt1 = float(len(t1))
        nt2 = float(len(t2))
        N_recip = 1./(nt1*nt1*nt2*nt2)
        GeneMap = root_mapper.GeneMap
        # ingroup/outgroup identification
        StoreSpeciesSets(tree, GeneMap, "inout_")
        # find all possible locations in the gene tree at which the root should be

        T = {True, }
        F = {False, }
        TF = set([True, False])
        for m in tree.traverse('postorder'):
            if m.is_leaf():
                if len(m.inout_up) == 1 and m.inout_up != m.inout_down:
                    # this is the unique root
                    return [m]
            else:
                if len(m.inout_up) == 1 and len(m.inout_down) == 1 and m.inout_up != m.inout_down:
                    # this is the unique root
                    return [m]
                nodes = m.get_children() if m.is_root() else [
                    m] + m.get_children()
                clades = [ch.inout_down for ch in nodes] if m.is_root() else (
                    [m.inout_up] + [ch.inout_down for ch in m.get_children()])
                # do we have the situation A | B or (A,B),S?
                if len(nodes) == 3:
                    if all([len(c) == 1 for c in clades]) and T in clades and F in clades:
                        # unique root
                        if clades.count(T) == 1:
                            return [nodes[clades.index(T)]]
                        else:
                            return [nodes[clades.index(F)]]
                    elif T in clades and F in clades:
                        #AB-(A,B) or B-(AB,A)
                        ab = [c == TF for c in clades]
                        i = ab.index(True)
                        roots_list.append(nodes[i])
                        sp_down = nodes[i].sp_down
                        sp_up = nodes[i].sp_up
#                        print(m)
                        scores_list.append(OutgroupIngroupSeparationScore(
                            sp_up, sp_down, sett1, sett2, N_recip, nt1, nt2))
                    elif clades.count(TF) >= 2:
                        # (A,A,A)-excluded, (A,A,AB)-ignore as want A to be bigest without including B, (A,AB,AB), (AB,AB,AB)
                        i = 0
                        roots_list.append(nodes[i])
                        sp_down = nodes[i].sp_down
                        sp_up = nodes[i].sp_up
#                        print(m)
                        scores_list.append(OutgroupIngroupSeparationScore(
                            sp_up, sp_down, sett1, sett2, N_recip, nt1, nt2))
                elif T in clades and F in clades:
                    roots_list.append(m)
                    scores_list.append(0)  # last choice
    # If we haven't found a unique root then use the scores for completeness of ingroup/outgroup to root
    if len(roots_list) == 0:
        return []  # This shouldn't occur
    return [sorted(zip(scores_list, roots_list), key=lambda x: x[0], reverse=True)[0][1]]


def WriteQfO2(orthologues_list_pairs_list, outfilename, qAppend=True):
    """ takes a list where each entry is a pair, (genes1, genes2), which are orthologues of one another
    """
    with open(outfilename, 'a' if qAppend else 'w') as outfile:
        for gs1, gs2, _, _ in orthologues_list_pairs_list:
            for sp1, genes1 in gs1.items():
                for sp2, genes2 in gs2.items():
                    for g1 in genes1:
                        for g2 in genes2:
                            outfile.write("%s_%s\t%s_%s\n" %
                                          (sp1, g1, sp2, g2))


def GetGeneToSpeciesMap(args):
    GeneToSpecies = GeneToSpecies_dash
    if args.separator and args.separator == "dot":
        GeneToSpecies = GeneToSpecies_dot
    elif args.separator and args.separator == "second_dash":
        GeneToSpecies = GeneToSpecies_secondDash
    elif args.separator and args.separator == "3rd_dash":
        GeneToSpecies = GeneToSpecies_3rdDash
    elif args.separator and args.separator == "hyphen":
        GeneToSpecies = GeneToSpecies_hyphen
    return GeneToSpecies


def OverlapSize(node, GeneToSpecies, suspect_genes):
    descendents = [{GeneToSpecies(l) for l in n.get_leaf_names()}.difference(
        suspect_genes) for n in node.get_children()]
    intersection = descendents[0].intersection(descendents[1])
    return len(intersection), intersection, descendents[0], descendents[1]


def ResolveOverlap(overlap, sp0, sp1, ch, tree, neighbours, GeneToSpecies, relOverlapCutoff=4):
    """
    Is an overlap suspicious and if so can it be resolved by identifying genes that are out of place?
    Args:
        overlap - the species with genes in both clades
        sp0 - the species below ch[0]
        sp1 - the species below ch[1]
        ch - the two child nodes
        tree - the gene tree
        neighbours - dictionary species->neighbours, where neighbours is a list of the sets of species observed at successive topological distances from the species
    Returns:
        qSuccess - has the overlap been resolved
        genes_removed - the out-of-place genes that have been removed so as to resolve the overlap

    Implementation:
        - The number of species in the overlap must be a 5th or less of the number of species in each clade - What if it's a single gene that's out of place? Won't make a difference then to the orthologs!
        - for each species with genes in both clades: the genes in one clade must all be more out of place (according to the 
          species tree) than all the gene from that species in the other tree
    """
    oSize = len(overlap)
    lsp0 = len(sp0)
    lsp1 = len(sp1)
    if (oSize == lsp0 or oSize == lsp1) or (relOverlapCutoff*oSize >= lsp0 and relOverlapCutoff*oSize >= lsp1):
        return False, []
    # The overlap looks suspect, misplaced genes?
    # for each species, we'd need to be able to determine that all genes from A or all genes from B are misplaced
    genes_removed = []
    nA_removed = 0
    nB_removed = 0
    qResolved = True
    for sp in overlap:
        A = [g for g in ch[0].get_leaf_names() if GeneToSpecies(g) == sp]
        B = [g for g in ch[1].get_leaf_names() if GeneToSpecies(g) == sp]
        A_levels = []
        B_levels = []
        for X, level in zip((A, B), (A_levels, B_levels)):
            for g in X:
                gene_node = tree & g
                r = gene_node.up
                nextSpecies = set([GeneToSpecies(gg)
                                   for gg in r.get_leaf_names()])
                # having a gene from the same species isn't enough?? No, but we add to the count I think.
                while len(nextSpecies) == 1:
                    r = r.up
                    nextSpecies = set([GeneToSpecies(gg)
                                       for gg in r.get_leaf_names()])
                nextSpecies.remove(sp)
                # get the level
                # the sum of the closest and furthest expected distance topological distance for the closest genes in the gene tree (based on species tree topology)
                neigh = neighbours[sp]
                observed = [neigh[nSp] for nSp in nextSpecies]
                level.append(min(observed) + max(observed))
        # if the clade is one step up the tree further way (min=max) then this gives +2. There's no way this is a problem
        qRemoveA = max(B_levels) + 2 < min(A_levels)
        qRemoveB = max(A_levels) + 2 < min(B_levels)
        if qRemoveA and relOverlapCutoff*oSize < len(sp0):
            nA_removed += len(A_levels)
            genes_removed.extend(A)
        elif qRemoveB and relOverlapCutoff*oSize < len(sp1):
            nB_removed += len(B_levels)
            genes_removed.extend(B)
        else:
            qResolved = False
            break
    if qResolved:
        return True, set(genes_removed)
    else:
        return False, set()


def GetRoot(tree, species_tree_rooted, GeneToSpecies):
    roots = GetRoots(tree, species_tree_rooted, GeneToSpecies)
    if len(roots) > 0:
        root_dists = [r.get_closest_leaf()[1] for r in roots]
        i, _ = max(enumerate(root_dists), key=operator.itemgetter(1))
        return roots[i]
    else:
        return None  # single species tree


def CheckAndRootTree(treeFN, species_tree_rooted, GeneToSpecies):
    """
    Check that the tree can be analysed and rooted
    Root tree
    Returns None if this fails, i.e. checks: exists, has more than one gene, can be rooted
    """
    # if (not os.path.exists(treeFN)) or os.stat(treeFN).st_size == 0:
    #     return None, False
    qHaveSupport = False
    try:
        tree = tree_lib.Tree(treeFN, format=2)
        qHaveSupport = True
    except:
        try:
            tree = tree_lib.Tree(treeFN)
        except:
            tree = tree_lib.Tree(treeFN, format=3)
    if len(tree) == 1:
        return None, False
    root = GetRoot(tree, species_tree_rooted, GeneToSpecies)
    if root == None:
        return None, False
    # Pick the first root for now
    if root != tree:
        tree.set_outgroup(root)
    return tree, qHaveSupport


def Orthologs_and_Suspect(ch, suspect_genes, misplaced_genes, SpeciesAndGene):
    """
    ch - the two child nodes that are orthologous
    suspect_genes - genes already identified as misplaced at lower levels
    misplaced_genes - genes identified as misplaced at this level
    Returns the tuple (o_0, o_1, os_0, os_1) where each element is a dictionary from species to genes from that species,
    the o are orthologs, the os are 'suspect' orthologs because the gene was previously identified as suspect
    """
    d = [defaultdict(list) for _ in range(2)]
    d_sus = [defaultdict(list) for _ in range(2)]
    for node, di, d_susi in zip(ch, d, d_sus):
        for g in [g for g in node.get_leaf_names() if g not in misplaced_genes]:
            sp, seq = SpeciesAndGene(g)
            if g in suspect_genes:
                d_susi[sp].append(seq)
            else:
                di[sp].append(seq)
    return d[0], d[1], d_sus[0], d_sus[1]


def GetOrthologues_from_tree(iog, tree, species_tree_rooted, GeneToSpecies, neighbours,
                             q_get_dups=False, qNoRecon=False):
    """ 
    Returns:
        orthologues 
        tree - Each node of the tree has two features added: dup (bool) and sp_node (str)
        suspect_genes - set
        duplications - list of (sp_node_name, genes0, genes1)
    """
    og_name = "OG%07d" % iog
    n_species = len(species_tree_rooted)
    # max_genes_dups = 5*n_species
    # max_genes_text = (">%d genes" % max_genes_dups,)
    qPrune = False
    SpeciesAndGene = SpeciesAndGene_lookup[GeneToSpecies]
    orthologues = []
    duplications = []
    if not qNoRecon:
        tree = Resolve(tree, GeneToSpecies)
    if qPrune:
        tree.prune(tree.get_leaf_names())
    if len(tree) == 1:
        return set(orthologues), tree, set(), duplications
    """ At this point need to label the tree nodes """
    iNode = 1
    tree.name = "n0"
    suspect_genes = set()
    empty_set = set()
    # preorder traverse so that suspect genes can be identified first, before their closer orthologues are proposed.
    # Tree resolution has already been performed using a postorder traversal
    for n in tree.traverse('preorder'):
        if n.is_leaf():
            continue
        if not n.is_root():
            n.name = "n%d" % iNode
            iNode += 1
        sp_present = None
        ch = n.get_children()
        if len(ch) == 2:
            oSize, overlap, sp0, sp1 = OverlapSize(
                n, GeneToSpecies, suspect_genes)
            sp_present = sp0.union(sp1)
            stNode = MRCA_node(species_tree_rooted, sp_present)
            n.add_feature("sp_node", stNode.name)
            if oSize != 0:
                # this should be moved to the tree resolution step. Except that doesn't use the species tree, so can't
                qResolved, misplaced_genes = ResolveOverlap(
                    overlap, sp0, sp1, ch, tree, neighbours, GeneToSpecies)
                # label the removed genes
                for g in misplaced_genes:
                    nn = tree & g
                    nn.add_feature("X", True)
            else:
                misplaced_genes = empty_set
            dup = oSize != 0 and not qResolved
            n.add_feature("dup", dup)
            if dup:
                if q_get_dups:
                    # genes0 = ch[0].get_leaf_names() if len(ch[0]) <= max_genes_dups else max_genes_text
                    # genes1 = ch[1].get_leaf_names() if len(ch[1]) <= max_genes_dups else max_genes_text
                    genes0 = ch[0].get_leaf_names()
                    genes1 = ch[1].get_leaf_names()
                    duplications.append((stNode.name, n.name, float(
                        oSize)/(len(stNode)), genes0, genes1))
            else:
                # sort out bad genes - no orthology for all the misplaced genes at this level (misplaced_genes).
                # For previous levels, (suspect_genes) have their orthologues written to suspect orthologues file
                orthologues.append(Orthologs_and_Suspect(
                    ch, suspect_genes, misplaced_genes, SpeciesAndGene))
                suspect_genes.update(misplaced_genes)
        elif len(ch) > 2:
            species = [{GeneToSpecies(l)
                        for l in n_.get_leaf_names()} for n_ in ch]
            all_species = set.union(*species)
            stNode = MRCA_node(species_tree_rooted, all_species)
            n.add_feature("sp_node", stNode.name)
            # should skip if everything below this is a single species, but should write out the duplications
            if len(all_species) == 1:
                # genes = n.get_leaf_names() if len(n) <= max_genes_dups else max_genes_text
                genes = n.get_leaf_names()
                duplications.append((stNode.name, n.name, 1., genes, []))
                n.add_feature("dup", True)
            else:
                dups = []
                for (n0, s0), (n1, s1) in itertools.combinations(zip(ch, species), 2):
                    if len(s0.intersection(s1)) == 0:
                        orthologues.append(Orthologs_and_Suspect(
                            (n0, n1), suspect_genes, empty_set, SpeciesAndGene))
                        dups.append(False)
                    else:
                        dups.append(True)
                if all(dups):
                    # genes = n.get_leaf_names() if len(n) <= max_genes_dups else max_genes_text
                    genes = n.get_leaf_names()
                    duplications.append((stNode.name, n.name, 1., genes, []))
                n.add_feature("dup", all(dups))
                # # if there are nodes below with same MRCA then dup (no HOGs) otherwise not dup (=> HOGS at this level)
                # descendant_nodes = [MRCA_node(species_tree_rooted, sp) for sp in species]
                # dup = any(down_sp_node == stNode for down_sp_node in descendant_nodes)
                # n.add_feature("dup", dup)
                # print(n.name + ": dup3")
    return orthologues, tree, suspect_genes, duplications


def Resolve(tree, GeneToSpecies):
    StoreSpeciesSets(tree, GeneToSpecies)
    for n in tree.traverse("postorder"):
        tree = resolve.resolve(n, GeneToSpecies)
    return tree


def GetSpeciesNeighbours(t):
    """
    Args: t = rooted species tree

    Returns:
    dict: species -> species_dict, such that species_dict: other_species -> toplogical_dist 
    """
    species = t.get_leaf_names()
    levels = {s: [] for s in species}
    for n in t.traverse('postorder'):
        if n.is_leaf():
            continue
        children = n.get_children()
        leaf_sets = [set(ch.get_leaf_names()) for ch in children]
        not_i = [set.union(*[l for j, l in enumerate(leaf_sets) if j != i])
                 for i in xrange(len(children))]
        for l, n in zip(leaf_sets, not_i):
            for ll in l:
                levels[ll].append(n)
    neighbours = {sp: {other: n for n, others in enumerate(
        lev) for other in others} for sp, lev in levels.items()}
    return neighbours


def RootAndGetOrthologues_from_tree(iog, tree_fn, species_tree_rooted, GeneToSpecies, neighbours, qWrite=False, qNoRecon=False):
    rooted_tree_ids, qHaveSupport = CheckAndRootTree(
        tree_fn, species_tree_rooted, GeneToSpecies)  # this can be parallelised easily
    if rooted_tree_ids is None:
        return
    orthologues, recon_tree, suspect_genes, dups = GetOrthologues_from_tree(
        iog, rooted_tree_ids, species_tree_rooted, GeneToSpecies, neighbours, qNoRecon=qNoRecon)
    if qWrite:
        directory = os.path.split(tree_fn)[0]
        WriteQfO2(orthologues, directory + "_Orthologues_M3/" +
                  os.path.split(tree_fn)[1], qAppend=False)


def RootTreeStandalone_Serial(trees_dir, species_tree_rooted_fn, GeneToSpecies, output_dir, qSingleTree):
    species_tree_rooted = tree_lib.Tree(species_tree_rooted_fn)
#    args_queue = mp.Queue()
    for treeFN in glob.glob(trees_dir + ("*" if qSingleTree else "/*")):
        if (not os.path.exists(treeFN)) or os.stat(treeFN).st_size == 0:
            return
        try:
            tree = tree_lib.Tree(treeFN)
        except:
            tree = tree_lib.Tree(treeFN, format=3)
        if len(tree) == 1:
            return
        root = GetRoot(tree, species_tree_rooted, GeneToSpecies)
        if root == None:
            return
        # Pick the first root for now
        if root != tree:
            tree.set_outgroup(root)
        tree.write(outfile=treeFN + ".rooted.txt")


def GetOrthologuesStandalone_Serial(trees_dir, species_tree_rooted_fn, GeneToSpecies, output_dir, qSingleTree):
    species_tree_rooted = tree_lib.Tree(species_tree_rooted_fn)
    neighbours = GetSpeciesNeighbours(species_tree_rooted)
#    args_queue = mp.Queue()
    for treeFn in glob.glob(trees_dir + ("*" if qSingleTree else "/*")):
        print(treeFn)
        # Now need to root the tree first
        RootAndGetOrthologues_from_tree(
            0, treeFn, species_tree_rooted, GeneToSpecies, neighbours)


def SortFile(fn, f_type):
    """
    Sort the contents of the file by the first column and save back to the same filename.
    Args:
        fn - filename
        f_type - o, x, h or d for orthologs, xenologs, hogs or duplications
    """
    def first_column_sort(s): return s.split("\t", 1)[0]
    if f_type == "h":
        # Need to renumber the hogs as the parallel numbering is incorrect
        with open(fn, csv_read_mode) as infile:
            try:
                header = next(infile)
            except StopIteration:
                return
            lines = []
            # remove incorrect HOG numbering
            for line in infile:
                lines.append(line.split("\t", 1)[-1])
            if len(lines) == 0:
                return
            hog_base = line.split(".", 1)[0]
        lines.sort(key=first_column_sort)
        with open(fn, csv_write_mode) as outfile:
            outfile.write(header)
            for ihog, l in enumerate(lines):
                outfile.write(hog_base + (".HOG%07d" % ihog) + "\t" + l)
    else:
        with open(fn, csv_read_mode) as infile:
            try:
                header = next(infile)
            except StopIteration:
                return
            lines = list(infile)
        lines.sort(key=first_column_sort)
        with open(fn, csv_write_mode) as outfile:
            outfile.write(header)
            outfile.write("".join(lines))


def GetOrthologues_from_phyldog_tree(iog, treeFN, GeneToSpecies, qWrite=False, dupsWriter=None, seqIDs=None, spIDs=None):
    """ if dupsWriter != None then seqIDs and spIDs must also be provided"""
    empty = set()
    orthologues = []
    if (not os.path.exists(treeFN)) or os.stat(treeFN).st_size == 0:
        return set(orthologues)
    tree = tree_lib.Tree(treeFN)
    if len(tree) == 1:
        return set(orthologues)
    """ At this point need to label the tree nodes """
    leaf_labels = dict()
    empty_dict = dict()
    for n in tree.traverse('preorder'):
        if n.is_leaf():
            leaf_labels[n.name] = ("n" + n.ND)
            continue
        else:
            n.name = "n" + n.ND
        ch = n.get_children()
        if len(ch) == 2:
            oSize, overlap, sp0, sp1 = OverlapSize(n, GeneToSpecies, empty)
            if n.Ev == "D":
                if dupsWriter != None:
                    sp_present = sp0.union(sp1)
                    stNode = "N" + n.S
                    if len(sp_present) == 1:
                        isSTRIDE = "Terminal"
                    else:
                        isSTRIDE = "Non-Terminal"
                    dupsWriter.writerow(["OG%07d" % iog, spIDs[stNode] if len(stNode) == 1 else stNode, n.name, "-", isSTRIDE, ", ".join(
                        [seqIDs[g] for g in ch[0].get_leaf_names()]), ", ".join([seqIDs[g] for g in ch[1].get_leaf_names()])])
            else:
                d0 = defaultdict(list)
                for g in ch[0].get_leaf_names():
                    sp, seq = g.split("_")
                    d0[sp].append(seq)
                d1 = defaultdict(list)
                for g in ch[1].get_leaf_names():
                    sp, seq = g.split("_")
                    d1[sp].append(seq)
                orthologues.append((d0, d1, empty_dict, empty_dict))
        elif len(ch) > 2:
            print("Non-binary node")
            print((n.get_leaf_names()))
    if qWrite:
        directory = os.path.split(treeFN)[0]
        WriteQfO2(orthologues, directory + "/../Orthologues_M3/" +
                  os.path.split(treeFN)[1], qAppend=False)
    return orthologues



"""
running diamond
diamond blastp --query Trinity.model.faa --max-target-seqs 10 --db /lustre/home/xuyuxing/Database/NCBI/nr/2020/nr.taxon.dmnd --evalue 1e-5 --out Trinity.model.faa.bls --outfmt 6 qseqid sseqid staxids pident length mismatch gapopen qstart qend sstart send evalue bitscore --threads 56

give me target species taxon
"""
from api.common.genome.blast import outfmt6_read_big
from lib.xuyuxing.evolution.taxonomy import read_tax_record_dict_db
from lib.common.os import get_file_dir


def detect_contaminate_from_diamond(bls_results_file, taxon_db_file, target_taxon):
    """
    bls_results_file = '/lustre/home/xuyuxing/Work/Orobanchaceae/Trans/clean_data/Tve/Trinity.model.faa.bls'
    taxon_db_file = '/lustre/home/xuyuxing/Database/NCBI/taxonomy/tax_xyx.db'

    target_taxon = 'Lamiales'
    """

    taxon_dict = read_tax_record_dict_db(taxon_db_file)

    try:
        target_taxon = str(int(target_taxon))
    except:
        t = [i for i in taxon_dict if taxon_dict[i].sci_name == target_taxon]
        if len(t) == 0:
            target_taxon = None
        else:
            target_taxon = t[0]

    contaminate_flag_dict = {}

    for query in outfmt6_read_big(bls_results_file, fieldname=["qseqid", "sseqid", "staxids", "pident", "length", "mismatch", "gapopen", "qstart", "qend", "sstart", "send", "evalue", "bitscore"], gzip_flag=False):
        high_sim_taxon = []
        for hit in query.hit:
            hit_taxon_list = hit.Hit_taxon_id
            for hsp in hit.hsp:
                if hsp.Hsp_identical_ratio > 0.98 and hsp.Hsp_align_len > 50:
                    high_sim_taxon.extend(hit_taxon_list)
        high_sim_taxon = list(set(high_sim_taxon))
        if "" in high_sim_taxon:
            high_sim_taxon.remove("")

        contaminate_flag = False
        if len(high_sim_taxon) > 0:
            contaminate_flag = True
            for t_id in high_sim_taxon:
                if t_id in taxon_dict:
                    t = taxon_dict[t_id]
                    if target_taxon in set(i[0] for i in t.get_lineage):
                        contaminate_flag = False

        contaminate_flag_dict[query.qDef] = contaminate_flag

    with open(get_file_dir(bls_results_file)+"/contamination.id", 'w') as f:
        for i in contaminate_flag_dict:
            if contaminate_flag_dict[i]:
                f.write(i+"\n")

    with open(get_file_dir(bls_results_file)+"/passed.id", 'w') as f:
        for i in contaminate_flag_dict:
            if contaminate_flag_dict[i] is False:
                f.write(i+"\n")

    



# sample.py
import ctypes
import os

# Try to locate the .so file in the same directory as this file
_file = 'libsample.so'
_path = os.path.join(*(os.path.split(__file__)[:-1] + (_file,)))
_mod = ctypes.cdll.LoadLibrary(_path)

# int gcd(int, int)
gcd = _mod.gcd
gcd.argtypes = (ctypes.c_int, ctypes.c_int)
gcd.restype = ctypes.c_int

# int in_mandel(double, double, int)
in_mandel = _mod.in_mandel
in_mandel.argtypes = (ctypes.c_double, ctypes.c_double, ctypes.c_int)
in_mandel.restype = ctypes.c_int

# int divide(int, int, int *)
_divide = _mod.divide
_divide.argtypes = (ctypes.c_int, ctypes.c_int, ctypes.POINTER(ctypes.c_int))
_divide.restype = ctypes.c_int


def divide(x, y):
    rem = ctypes.c_int()
    quot = _divide(x, y, rem)

    return quot, rem.value


# void avg(double *, int n)
# Define a special type for the 'double *' argument
class DoubleArrayType:
    def from_param(self, param):
        typename = type(param).__name__
        if hasattr(self, 'from_' + typename):
            return getattr(self, 'from_' + typename)(param)
        elif isinstance(param, ctypes.Array):
            return param
        else:
            raise TypeError("Can't convert %s" % typename)

    # Cast from array.array objects
    def from_array(self, param):
        if param.typecode != 'd':
            raise TypeError('must be an array of doubles')
        ptr, _ = param.buffer_info()
        return ctypes.cast(ptr, ctypes.POINTER(ctypes.c_double))

    # Cast from lists/tuples
    def from_list(self, param):
        val = ((ctypes.c_double) * len(param))(*param)
        return val

    from_tuple = from_list

    # Cast from a numpy array
    def from_ndarray(self, param):
        return param.ctypes.data_as(ctypes.POINTER(ctypes.c_double))


DoubleArray = DoubleArrayType()
_avg = _mod.avg
_avg.argtypes = (DoubleArray, ctypes.c_int)
_avg.restype = ctypes.c_double


def avg(values):
    return _avg(values, len(values))


# struct Point { }
class Point(ctypes.Structure):
    _fields_ = [('x', ctypes.c_double),
                ('y', ctypes.c_double)]


# double distance(Point *, Point *)
distance = _mod.distance
distance.argtypes = (ctypes.POINTER(Point), ctypes.POINTER(Point))
distance.restype = ctypes.c_double



from lib.common.genome.genome_feature2 import GenomeFeature, ChrLoci, write_gff_file
from api.xuyuxing.genome.repeatmasker import repeatmasker_parser


def RepeatMasker2Gff3_main(args):
    repeat_info = repeatmasker_parser(args.repeatmasker_out)

    gf_list = []
    for i in repeat_info:
        for j in repeat_info[i]:
            for k in repeat_info[i][j].case_list:
                gf = GenomeFeature(id="repeatmasked_%s" % k.ID,
                                   type=k.repeat_class,
                                   chr_loci=ChrLoci(
                                       chr_id=k.q_name, start=k.q_start, end=k.q_end, strand=k.strand),
                                   qualifiers={
                                       'SW_score': k.SW_score,
                                       'perc_div': k.perc_div,
                                       'perc_del': k.perc_del,
                                       'perc_ins': k.perc_ins,
                                       'repeat_name': k.repeat_name,
                                       'repeat_class': k.repeat_class,
                                       'r_start': k.r_start,
                                       'r_end': k.r_end,
                                       'r_length': k.r_length,
                                       'star': k.star,
                                   })
                gf_list.append(gf)

    write_gff_file(gf_list, args.output_gff, sort=True)


if __name__ == '__main__':
    class abc():
        pass

    args = abc()
    args.repeatmasker_out = "/lustre/home/xuyuxing/Database/Orobanche/annotation/repeat/RepeatMasker.out/Ocu.genome.nt.1.0.fasta.out"
    args.output_gff = "/lustre/home/xuyuxing/Database/Orobanche/annotation/repeat/RepeatMasker.out/Ocu.genome.nt.1.0.fasta.gff3"



import lib.common.os
from lib.common.fileIO import tsv_file_dict_parse
from lib.xuyuxing.evolution.taxonomy import read_tax_record_dict_db
import lib.xuyuxing.base.base_function as bf
import xml.etree.ElementTree as ET
import re
import os
import time
from ftplib import FTP


class GenomeRecord(object):
    """
    Genome_record is a class for each record in genome database.
    """

    def __init__(self, db_access_id, taxon_id, download_info=None, db_name=None, other_info=None):
        self.db_access_id = db_access_id
        self.taxon_id = taxon_id
        self.db_name = db_name
        self.other_info = other_info
        self.download_info = download_info


def jgi_Portal_ID_get(Portal_ID_raw):
    return re.findall("\"(.*)\"", Portal_ID_raw.split(",")[1])[0]


def jgi_Taxonomy_ID_get(genome_projects_record):
    """
    some of record in JGI genome_project.csv file have very long taxonomy id,
    because they are repeated by many Sequencing Project ID
    """
    seq_project_count = len(re.findall(r'(\S+)', genome_projects_record["Sequencing Project ID"]))
    tax_id_raw_len = len(genome_projects_record["Taxonomy ID"])

    if seq_project_count == 0:
        return genome_projects_record["Taxonomy ID"]

    if tax_id_raw_len % seq_project_count == 0:
        tax_id_len = int(tax_id_raw_len / seq_project_count)
        split_list = re.findall('.{%d}' % tax_id_len, genome_projects_record["Taxonomy ID"])
        if len(list(set(split_list))) == 1:
            return split_list[0]
        else:
            return genome_projects_record["Taxonomy ID"]
    return genome_projects_record["Taxonomy ID"]


def jgi_pull_info(user_name, passwd, cookie_path, output_xml, Portal_ID, directly_download):
    cmd_string = "curl 'https://genome.jgi.doe.gov/portal/ext-api/downloads/get-directory?organism=%s' -b %s > %s" % (
        Portal_ID, cookie_path, output_xml)

    if directly_download is False:
        return cmd_string

    lib.common.os.cmd_run(cmd_string)

    if os.path.exists(output_xml) and os.path.getsize(output_xml) != 0:
        return output_xml
    else:
        login_string = "curl 'https://signon.jgi.doe.gov/signon/create' --data-urlencode 'login=%s' --data-urlencode 'password=%s' -c %s > /dev/null" % (
            user_name, passwd, cookie_path)
        lib.common.os.cmd_run(login_string)
        lib.common.os.cmd_run(cmd_string)
        return output_xml


def jgi_pull_file(user_name, passwd, cookie_path, output_file, want_url, directly_download):
    cmd_string = "curl '%s' -b %s > %s" % (
        want_url.replace("&amp;", "&"), cookie_path, output_file)

    if directly_download is False:
        return cmd_string

    lib.common.os.cmd_run(cmd_string)

    if os.path.exists(output_file) and os.path.getsize(output_file) != 0:
        return output_file
    else:
        login_string = "curl 'https://signon.jgi.doe.gov/signon/create' --data-urlencode 'login=%s' --data-urlencode 'password=%s' -c %s > /dev/null" % (
            user_name, passwd, cookie_path)
        lib.common.os.cmd_run(login_string)
        lib.common.os.cmd_run(cmd_string)
        return output_file


def jgi_files_download(ftp_list, output_list, user_name, passwd, cookie_path):
    for tmp_index in range(len(ftp_list)):
        ftp_url = ftp_list[tmp_index]
        output_file = output_list[tmp_index]

        jgi_pull_file(user_name, passwd, cookie_path, output_file, ftp_url, True)


def jgi_uniq_file_record(file_list):
    def good_choose(file_list_tmp):
        for file_record in file_list_tmp:
            file_url, file_name, file_md5, tag_tmp, stat, file_type, file_size = file_record
            match_url = re.match(r'.*_JAMO.*', file_url)
            if match_url:
                return file_record
        return file_list_tmp[0]

    file_dir = {}
    for file_record in file_list:
        file_url, file_name, file_md5, tag_tmp, stat, file_type, file_size = file_record
        if not file_name in file_dir:
            file_dir[file_name] = []
        file_dir[file_name].append(file_record)

    output_list = []
    for file_name in file_dir:
        file_list = file_dir[file_name]
        output_list.append(good_choose(file_list))

    return output_list


def jgi_metadata_xml_read(xml_file):
    tree = ET.parse(xml_file)
    root = tree.getroot()

    file_info_dict = {}
    md5_info_dict = {}
    url_info_dict = {}
    for file_tmp in root.iter("file"):
        if "filename" in file_tmp.attrib:
            file_name = file_tmp.attrib["filename"]
        else:
            continue

        if "url" in file_tmp.attrib:
            match_url = re.match(r'.*&url=(.*)', file_tmp.attrib["url"])
            if match_url:
                file_url = "https://genome.jgi.doe.gov/portal" + match_url.group(1)
            else:
                file_url = "https://genome.jgi.doe.gov" + file_tmp.attrib["url"]
            url_info_dict[file_name] = file_url
        else:
            continue

        if "md5" in file_tmp.attrib:
            file_md5 = file_tmp.attrib["md5"]
            md5_info_dict[file_name] = file_md5
        else:
            file_md5 = ""

        if "sizeInBytes" in file_tmp.attrib:
            file_size = file_tmp.attrib["sizeInBytes"]
        else:
            file_size = ""

        file_type = ""
        match_gff = re.match(r"^(\S+)_GeneCatalog_genes_(\d+)\.gff.gz", file_name)
        if match_gff:
            tag_tmp = match_gff.groups(1)[0]
            version_tmp = match_gff.groups(1)[1]
            file_type = "gff_file"
        match_cds = re.match(r"^(\S+)_GeneCatalog_CDS_(\d+)\.fasta.gz", file_name)
        if match_cds:
            tag_tmp = match_cds.groups(1)[0]
            version_tmp = match_cds.groups(1)[1]
            file_type = "cds_file"
        match_pt = re.match(r"^(\S+)_GeneCatalog_proteins_(\d+)\.aa.fasta.gz", file_name)
        if match_pt:
            tag_tmp = match_pt.groups(1)[0]
            version_tmp = match_pt.groups(1)[1]
            file_type = "pt_file"
        match_genome = re.match(r"^(\S+)_AssemblyScaffolds.fasta.gz", file_name)
        if match_genome:
            tag_tmp = match_genome.groups(1)[0]
            file_type = "genome_file"

        if file_type == "":
            continue

        if tag_tmp not in file_info_dict:
            file_info_dict[tag_tmp] = {}
            file_info_dict[tag_tmp]["genome_file"] = None
            file_info_dict[tag_tmp]["annotation"] = {}
        if file_type == "genome_file":
            file_info_dict[tag_tmp]["genome_file"] = (
                file_url, file_name, file_md5, tag_tmp, "", "genome_file", file_size)
        else:
            if version_tmp not in file_info_dict[tag_tmp]["annotation"]:
                file_info_dict[tag_tmp]["annotation"][version_tmp] = []
            file_info_dict[tag_tmp]["annotation"][version_tmp].append(
                (file_url, file_name, file_md5, tag_tmp, "", file_type, file_size))
            file_info_dict[tag_tmp]["annotation"][version_tmp] = jgi_uniq_file_record(
                file_info_dict[tag_tmp]["annotation"][version_tmp])

    # choose newest version

    for tag_tmp in file_info_dict:
        file_info = {
            "genome_file": None,
            "gff_file": None,
            "cds_file": None,
            "pt_file": None
        }

        genome_file_info = file_info_dict[tag_tmp]["genome_file"]
        if not genome_file_info is None:

            g_file_name = genome_file_info[1]

            if g_file_name in md5_info_dict:
                g_file_md5 = md5_info_dict[g_file_name]
            else:
                g_file_md5 = None

            if g_file_name in url_info_dict:
                g_file_url = url_info_dict[g_file_name]
            else:
                g_file_url = None

            file_info["genome_file"] = (g_file_name, g_file_url, g_file_md5)

        version_keys = list(file_info_dict[tag_tmp]['annotation'].keys())

        if len(version_keys) != 0:

            version_keys = [int(i) for i in version_keys]
            newest_version = str(sorted(version_keys, reverse=True)[0])

            for i in file_info_dict[tag_tmp]["annotation"][newest_version]:
                file_name = i[1]

                if file_name in md5_info_dict:
                    file_md5 = md5_info_dict[file_name]
                else:
                    file_md5 = None

                if file_name in url_info_dict:
                    file_url = url_info_dict[file_name]
                else:
                    file_url = None

                file_info[i[5]] = (file_name, file_url, file_md5)

        file_info_dict[tag_tmp] = file_info

    return file_info_dict


def get_ncbi_md5_list(ftp_dir, tmp_file="/tmp/tmp_md5.txt"):
    """
    ftp_dir = '/genomes/all/GCA/000/001/215/GCA_000001215.4_Release_6_plus_ISO1_MT'
    """

    ftp_dir = re.sub(r'ftp://ftp.ncbi.nlm.nih.gov', '', ftp_dir)

    ftp = FTP()
    # ftp.set_debuglevel(2)
    ftp.connect("ftp.ncbi.nlm.nih.gov")
    ftp.login()

    try:
        fp = open(tmp_file, 'wb')
        bufsize = 1024
        ftp.retrbinary('RETR ' + ftp_dir + "/md5checksums.txt", fp.write, bufsize)
        fp.close()

        md5_info_dict = {}
        with open(tmp_file, 'r') as f:
            for each_line in f:
                each_line = each_line.strip()
                info = each_line.split()
                file_name = re.sub(r'^\./', '', info[1])
                md5_string = info[0]
                md5_info_dict[file_name] = md5_string

        os.remove(tmp_file)

        return md5_info_dict
    except:
        return "error"


def get_many_ncbi_md5_list(ftp_dir_list, md5sum_huge_file, tmp_dir="/tmp/tmp_md5"):
    import time
    from lib.xuyuxing.base.common_command import log_print

    lib.common.os.mkdir(tmp_dir)
    #
    # with open(tmp_dir + "/ftp_md5_file.txt", 'w') as f:
    #     num = 0
    #     for i in ftp_dir_list:
    #         num = num + 1
    #         rename_md5 = tmp_dir + "/md5_%d.txt" % (num)
    #         f.write("wget -O %s %s/md5checksums.txt\n" % (rename_md5, i))

    log_print("\t\tBegin: download md5sum file for %d ncbi genome record" % len(ftp_dir_list))
    start_time = time.time()

    ftp = FTP()
    # ftp.set_debuglevel(2)
    ftp.connect("ftp.ncbi.nlm.nih.gov")
    ftp.login()

    num = 0
    rename_md5_list = []
    for ftp_dir in ftp_dir_list:

        round_time = time.time()
        if round_time - start_time > 10:
            log_print("\t\t\t%d/%d" % (num, len(ftp_dir_list)))
            start_time = round_time

        ftp_dir = re.sub(r'ftp://ftp.ncbi.nlm.nih.gov', '', ftp_dir)
        num += 1
        rename_md5 = tmp_dir + "/md5_%d.txt" % (num)
        failed_flag = True
        retry = 0
        while failed_flag and retry < 5:
            try:
                fp = open(rename_md5, 'wb')
                bufsize = 1024
                ftp.retrbinary('RETR ' + ftp_dir + "/md5checksums.txt", fp.write, bufsize)
                fp.close()
                if os.path.exists(rename_md5) and os.path.getsize(rename_md5) != 0:
                    rename_md5_list.append(rename_md5)
                    failed_flag = False
            except:
                ftp = FTP()
                # ftp.set_debuglevel(2)
                ftp.connect("ftp.ncbi.nlm.nih.gov")
                ftp.login()
                failed_flag = True
            retry += 1

    ftp.close()

    log_print("\t\tEnd: download all md5sum file for %d ncbi genome record" % len(ftp_dir_list))

    with open(md5sum_huge_file, 'w') as fo:
        for file_tmp in rename_md5_list:
            with open(file_tmp, 'r') as fi:
                for each_line in fi:
                    fo.write(each_line)

    return md5sum_huge_file


def ncbi_files_download(ftp_list, output_list):
    import time
    from lib.xuyuxing.base.common_command import log_print

    log_print("\t\tBegin: download %d files from ncbi database" % len(ftp_list))
    start_time = time.time()

    ftp = FTP()
    # ftp.set_debuglevel(2)
    ftp.connect("ftp.ncbi.nlm.nih.gov")
    ftp.login()

    num = 0
    rename_md5_list = []
    for tmp_index in range(len(ftp_list)):
        ftp_url = ftp_list[tmp_index]
        output_file = output_list[tmp_index]

        round_time = time.time()
        num += 1
        if round_time - start_time > 10:
            log_print("\t\t\t%d/%d" % (num, len(ftp_list)))
            start_time = round_time

        ftp_url = re.sub(r'ftp://ftp.ncbi.nlm.nih.gov', '', ftp_url)

        failed_flag = True
        retry = 0
        while failed_flag and retry < 5:
            try:
                fp = open(output_file, 'wb')
                bufsize = 1024
                ftp.retrbinary('RETR ' + ftp_url, fp.write, bufsize)
                fp.close()
                if os.path.exists(output_file) and os.path.getsize(output_file) != 0:
                    rename_md5_list.append(output_file)
                    failed_flag = False
            except:
                ftp = FTP()
                # ftp.set_debuglevel(2)
                ftp.connect("ftp.ncbi.nlm.nih.gov")
                ftp.login()
                failed_flag = True
            retry += 1

    ftp.close()

    log_print("\t\tEnd: download all md5sum file for %d ncbi genome record" % len(ftp_list))


def genome_record_exclude_by_taxon(record_dict, tax_sqldb_file, taxon_filter):
    # exclude taxon by filter
    used_taxon_id_list = list(set([record_dict[i].taxon_id for i in record_dict]))
    tax_record_db = read_tax_record_dict_db(tax_sqldb_file, tuple(used_taxon_id_list))

    rec_id_list = list(record_dict.keys())
    for i in rec_id_list:
        if record_dict[i].taxon_id not in tax_record_db:
            del record_dict[i]
        else:
            lineage_id_list = [i[0] for i in tax_record_db[record_dict[i].taxon_id].get_lineage]
            if len(set(lineage_id_list) & set(taxon_filter)) == 0:
                del record_dict[i]

    return record_dict


def build_data_index(input_info_list, database_name, tax_sqldb_file, taxon_filter):
    """
    most database will give a index file to tell me, how many genome they have
    """

    if database_name == 'ncbi':
        """
        "assembly_summary_refseq_file" and "assembly_summary_genbank_file" as input
        """

        assembly_summary_refseq_file, assembly_summary_genbank_file, md5sum_huge_file = input_info_list

        col_name = ["assembly_accession", "bioproject", "biosample", "wgs_master", "refseq_category", "taxid",
                    "species_taxid", "organism_name", "infraspecific_name", "isolate", "version_status",
                    "assembly_level", "release_type", "genome_rep", "seq_rel_date", "asm_name", "submitter",
                    "gbrs_paired_asm", "paired_asm_comp", "ftp_path", "excluded_from_refseq",
                    "relation_to_type_material"]

        record_dict = {}

        parse_file_db = tsv_file_dict_parse(assembly_summary_refseq_file, fieldnames=col_name)
        for i in parse_file_db:
            parse_file_db[i]['group'] = "refseq"
            gr_tmp = GenomeRecord(parse_file_db[i]['assembly_accession'], parse_file_db[i]["species_taxid"],
                                  None, "ncbi", parse_file_db[i])
            record_dict[parse_file_db[i]['assembly_accession']] = gr_tmp

        parse_file_db = tsv_file_dict_parse(assembly_summary_genbank_file, fieldnames=col_name)
        for i in parse_file_db:
            if parse_file_db[i]['paired_asm_comp'] == "na" and parse_file_db[i]['excluded_from_refseq'] == "" and \
                    parse_file_db[i]['ftp_path'] != "na":
                parse_file_db[i]['group'] = "genbank"
                gr_tmp = GenomeRecord(parse_file_db[i]['assembly_accession'], parse_file_db[i]["species_taxid"],
                                      None, "ncbi", parse_file_db[i])
                record_dict[parse_file_db[i]['assembly_accession']] = gr_tmp

        # exclude taxon by filter
        record_dict = genome_record_exclude_by_taxon(record_dict, tax_sqldb_file, taxon_filter)

        # add download info
        if not os.path.exists(md5sum_huge_file):
            ftp_dir_list = [record_dict[i].other_info['ftp_path'] for i in record_dict]
            # ftp_dir_list = ftp_dir_list[0:100]
            get_many_ncbi_md5_list(ftp_dir_list, md5sum_huge_file, tmp_dir="/tmp/tmp_md5")

        md5_info_dict = {}
        with open(md5sum_huge_file, 'r') as f:
            for each_line in f:
                each_line = each_line.strip()
                info = each_line.split()
                file_name = re.sub(r'^\./', '', info[1])
                md5_string = info[0]
                md5_info_dict[file_name] = md5_string

        good_num = 0
        bad_num = 0
        for rec_id in record_dict:
            long_acc = record_dict[rec_id].other_info['ftp_path'].split('/')[-1]

            ftp_path = record_dict[rec_id].other_info['ftp_path']

            file_info = {
                "genome_file": long_acc + "_genomic.fna.gz",
                "gff_file": long_acc + "_genomic.gff.gz",
                "cds_file": long_acc + "_cds_from_genomic.fna.gz",
                "pt_file": long_acc + "_translated_cds.faa.gz"
            }

            for i in file_info:
                if file_info[i] in md5_info_dict:
                    file_tuple = (file_info[i], ftp_path + "/" + file_info[i], md5_info_dict[file_info[i]])
                    file_info[i] = file_tuple
                    good_num += 1
                else:
                    file_info[i] = None
                    bad_num += 1

            record_dict[rec_id].download_info = file_info

        return record_dict

    elif database_name == 'jgi':
        """
        "genome-projects.csv" and xml file like "fungi.xml" as input
        """

        # read "fungi.xml" like file for record download info
        metadata_xml_path = input_info_list[1]
        file_info_dict = jgi_metadata_xml_read(metadata_xml_path)

        # read "genome-projects.csv" for record speci info

        col_name = ["Project Name", "Principal Investigator", "Scientific Program", "Product Name", "Status",
                    "Status Date", "User Program", "Proposal", "JGI Project Id", "Taxonomy ID", "NCBI Project Id",
                    "Genbank", "ENA", "SRA", "Sequencing Project ID", "Analysis project ID", "Project Manager",
                    "Portal ID", "IMG Portal", "Mycocosm Portal", "Phytozome Portal"]

        genome_projects_csv_file = input_info_list[0]

        record_dict = {}

        parse_file_db = tsv_file_dict_parse(genome_projects_csv_file, fieldnames=col_name, seq=",", ignore_head_num=1)
        for i in parse_file_db:
            record_id = jgi_Portal_ID_get(parse_file_db[i]["Portal ID"])
            record_species_taxid = jgi_Taxonomy_ID_get(parse_file_db[i])

            if record_species_taxid == "":
                continue

            if record_id not in file_info_dict:
                continue

            gr_tmp = GenomeRecord(record_id, record_species_taxid, file_info_dict[record_id], 'jgi', parse_file_db[i])
            record_dict[record_id] = gr_tmp

        # exclude taxon by filter
        record_dict = genome_record_exclude_by_taxon(record_dict, tax_sqldb_file, taxon_filter)

        return record_dict


def class_genome_record_by_rank(record_dict, taxon_rank, tax_sqldb_file):
    used_taxon_id_list = list(set([record_dict[i].taxon_id for i in record_dict]))
    tax_record_db = read_tax_record_dict_db(tax_sqldb_file, tuple(used_taxon_id_list))

    record_in_rank = {}
    record_in_rank['other'] = []
    for gr_id in record_dict:
        gr_tmp = record_dict[gr_id]
        gr_taxon = tax_record_db[gr_tmp.taxon_id]
        if hasattr(gr_taxon, taxon_rank):
            taxonomic_rank_tmp = getattr(gr_taxon, taxon_rank)
            if taxonomic_rank_tmp not in record_in_rank:
                record_in_rank[taxonomic_rank_tmp] = []
            record_in_rank[taxonomic_rank_tmp].append(gr_tmp)
        else:
            record_in_rank['other'].append(gr_tmp)

    return record_in_rank


if __name__ == '__main__':
    from lib.common.os import mkdir
    from lib.xuyuxing.evolution.taxonomy import build_taxon_database, store_tax_record_into_sqlite

    # normal
    work_dir = "/lustre/home/xuyuxing/Work/Gel/HGT"
    mkdir(work_dir, True)

    path_for_taxonomy = "/lustre/home/xuyuxing/Work/Gel/HGT/taxdump"
    tax_db_file = "/lustre/home/xuyuxing/Work/Gel/HGT/taxdump/tax_xyx.db"

    tax_record_dict = build_taxon_database(path_for_taxonomy)
    store_tax_record_into_sqlite(tax_record_dict, tax_db_file)

    # build record index
    # ncbi
    """
    assembly_summary file ("assembly_summary_genbank.txt" and "assembly_summary_refseq.txt") and genome size file from NCBI.
    ftp://ftp.ncbi.nlm.nih.gov/genomes/ASSEMBLY_REPORTS/assembly_summary_refseq.txt
    ftp://ftp.ncbi.nlm.nih.gov/genomes/ASSEMBLY_REPORTS/assembly_summary_genbank.txt
    ftp://ftp.ncbi.nih.gov/pub/taxonomy/taxdump.tar.gz (need decompressed)
    ftp://ftp.ncbi.nlm.nih.gov/genomes/ASSEMBLY_REPORTS/species_genome_size.txt.gz (need decompressed)
    """

    assembly_summary_refseq_file = '/lustre/home/xuyuxing/Work/Gel/HGT/assembly_summary_refseq.txt'
    assembly_summary_genbank_file = '/lustre/home/xuyuxing/Work/Gel/HGT/assembly_summary_genbank.txt'
    md5sum_huge_file = '/lustre/home/xuyuxing/Work/Gel/HGT/md5sum_huge_file.txt'

    ncbi_fungi_record_dict = build_data_index(
        [assembly_summary_refseq_file, assembly_summary_genbank_file, md5sum_huge_file], 'ncbi', tax_db_file, ['4751'])

    # jgi
    jgi_user_name = "xuyuxing14@mails.ucas.ac.cn"
    jgi_passwd = "HGTrunning123"
    metadata_key_words = "fungal-program-all"
    genome_projects_csv_file = "/lustre/home/xuyuxing/Work/Gel/HGT/genome-projects.csv"

    jgi_dir = work_dir + "/jgi"
    mkdir(jgi_dir, True)
    cookie_path = work_dir + "/jgi/cookies"
    # metadata_xml_path = work_dir + "/jgi/fungi.xml"
    metadata_xml_path = "/lustre/home/xuyuxing/Work/Gel/HGT/jgi/fungal-program-all.xml"

    # get fungi.xml
    jgi_pull_info(jgi_user_name, jgi_passwd, cookie_path, metadata_xml_path, "fungal-program-all", True)

    # build index
    jgi_fungi_record_dict = build_data_index([genome_projects_csv_file, metadata_xml_path], 'jgi', tax_db_file,
                                             ['4751'])

    # merge ncbi and jgi database
    from lib.xuyuxing.base.common_command import merge_dict

    record_dict = merge_dict([ncbi_fungi_record_dict, jgi_fungi_record_dict], False)

    record_in_genus = class_genome_record_by_rank(record_dict, 'genus', tax_db_file)

    # download files
    Am_list = record_in_genus['47424']

    Am_jgi_data = {i.db_access_id: i.download_info['pt_file'] for i in Am_list if
                   i.db_name == 'jgi' and not i.download_info['pt_file'] is None}
    Am_ncbi_data = {i.db_access_id: i.download_info['pt_file'] for i in Am_list if
                    i.db_name == 'ncbi' and not i.download_info['pt_file'] is None}

    ## download ncbi data
    ncbi_data_dir = "/lustre/home/xuyuxing/Work/Gel/HGT/ncbi_am"
    for i in Am_ncbi_data:
        file_name, download_url, md5sum = Am_ncbi_data[i]
        store_file = ncbi_data_dir + "/" + file_name
        Am_ncbi_data[i] = (file_name, download_url, md5sum, store_file)

    Am_ncbi_url_list = [Am_ncbi_data[i][1] for i in Am_ncbi_data]
    Am_ncbi_file_list = [Am_ncbi_data[i][3] for i in Am_ncbi_data]

    ncbi_files_download(Am_ncbi_url_list, Am_ncbi_file_list)

    ## download jgi data
    jgi_data_dir = "/lustre/home/xuyuxing/Work/Gel/HGT/jgi_am"
    for i in Am_jgi_data:
        file_name, download_url, md5sum = Am_jgi_data[i]
        store_file = jgi_data_dir + "/" + file_name
        Am_jgi_data[i] = (file_name, download_url, md5sum, store_file)

    Am_jgi_url_list = [Am_jgi_data[i][1] for i in Am_jgi_data]
    Am_jgi_file_list = [Am_jgi_data[i][3] for i in Am_jgi_data]

    jgi_user_name = "xuyuxing14@mails.ucas.ac.cn"
    jgi_passwd = "HGTrunning123"
    cookie_path = "/lustre/home/xuyuxing/Work/Gel/HGT/jgi/cookies"
    jgi_files_download(Am_jgi_url_list, Am_jgi_file_list, jgi_user_name, jgi_passwd, cookie_path)



import networkx as nx
from itertools import combinations
from lib.common.util import dict_key_value_transpose

__author__ = 'Yuxing Xu'

def merge_same_element_set_with_group_id(input_dict):
    """
    merge sets which have at least one same element
    :param input_dict:
            input_dict = {
                "a" : [1,2,3,4,5],
                "b" : [10,2,30,40,50],
                "c" : [100,200,300,40,500],
                "d" : [900,1000]
                }
    :return: output_list = [(a,b),(c)]
    """

    def one_step(input_dict):
        input_hash = dict_key_value_transpose(input_dict)
        change_flag = 0
        for element in input_hash:
            if len(input_hash[element]) > 1:
                all_group_not_deleted_flag = 1
                for group in input_hash[element]:
                    if group not in input_dict:
                        all_group_not_deleted_flag = 0
                        break

                if all_group_not_deleted_flag:

                    change_flag = 1

                    temp_group_id = []
                    for i in input_hash[element]:
                        if isinstance(i, tuple):
                            for j in i:
                                temp_group_id.append(j)
                        else:
                            temp_group_id.append(i)
                    temp_group_id = tuple(set(temp_group_id))

                    temp_group_element = []
                    for group in input_hash[element]:
                        temp_group_element.extend(input_dict[group])
                    temp_group_element = list(set(temp_group_element))

                    for group in input_hash[element]:
                        del input_dict[group]

                    input_dict[temp_group_id] = temp_group_element

                else:
                    continue

        return change_flag, input_dict

    change_flag = 1
    while change_flag:
        change_flag, input_dict = one_step(input_dict)
        print(len(input_dict))

    return input_dict


def merge_same_element_set_with_group_id_by_graphs(input_dict):
    """
    merge sets which have at least one same element
    based on Connected component in graphs
    :param input_dict:
            input_dict = {
                "a" : [1,2,3,4,5],
                "b" : [10,2,30,40,50],
                "c" : [100,200,300,40,500],
                "d" : [900,1000]
                }
    :return: output_list = [(a,b,c),(d)]
    """
    G = nx.Graph()
    G.add_nodes_from(input_dict.keys())
    input_hash = dict_key_value_transpose(input_dict)
    for element in input_hash:
        if len(input_hash[element]) > 1:
            G.add_edges_from(combinations(input_hash[element], 2))

    output_list = []
    for sub_graph in (G.subgraph(c) for c in nx.connected_components(G)):
        output_list.append(tuple(list(sub_graph.nodes)))

    output_dir = {}
    for i in output_list:
        element_list = []
        for j in i:
            element_list.extend(input_dict[j])
        output_dir[i] = list(set(element_list))

    return output_dir


def merge_same_element_set(input_lol):
    """
    merge sets which have at least one same element
    :param input_lol:
            input_lol = [[1,2,3,4,5],[10,2,30,40,50],[100,200,300,40,500],[900,1000]]
    :return: output_list = [[1,2,3,4,5,10,30,40,50,100,200,300,500],[900,1000]]
    """
    num = 0
    input_dict = {}
    for i in input_lol:
        input_dict[num] = i
        num = num + 1

    # output_dict = merge_same_element_set_with_group_id(input_dict)
    output_dict = merge_same_element_set_with_group_id_by_graphs(input_dict)

    output_list = []
    for i in output_dict:
        output_list.append(output_dict[i])

    return output_list


def merge_subset(set_list):
    """
    remove set which is subset of other set

    :param set_list: [{1,2},{1,2,3},{1,2},{1,2,4},{1,2,3},{1,2},{1,4},{1,2,3},{1,2,3,5}]
    :return: [{1,2,3,5},{1,2,4}]
    """

    set_list = [set(i) for i in set_list]

    want_remove = []
    for i in range(len(set_list)):
        set_i = set_list[i]
        for j in range(i, len(set_list)):
            set_j = set_list[j]
            if i == j:
                continue
            if set_i & set_j == set_i:
                want_remove.append(i)
            elif set_i & set_j == set_j:
                want_remove.append(j)

    want_remove = list(set(want_remove))

    return [set_list[i] for i in range(len(set_list)) if not i in want_remove]


def point_break(all_list, point_list):
    """
    point_list = [260, 264, 267, 281, 282, 289, 272, 290, 291, 292, 293, 294]
    all_list = [260, 261, 262, 263, 264, 265, 266, 267, 268, 269, 270, 271, 272, 273, 274, 275, 276, 277, 278, 279, 280, 281, 282, 283, 284, 285, 286, 287, 288, 289, 290, 291, 292, 293, 294]
    """

    output_lol = []
    last_range = []
    for i in all_list:
        if i in point_list:
            if len(last_range) != 0:
                output_lol.append(last_range)
            last_range = []
        else:
            last_range.append(i)
    if len(last_range) != 0:
        output_lol.append(last_range)

    return output_lol    


import string
import re

class BjobsCommand():
    def __init__(self):
        pass

#    def seqname_short(self):
#        name_short = re.search('^(\S+)', self.seqname).group(1)
#        return name_short

def read_bjobs_l(file_name):
    with open(file_name, 'r') as f:
        all_text = f.read()
        info = string.split(all_text, '------------------------------------------------------------------------------')
        while '' in info:
            info.remove('')
        dict_output={}
        for i in info:
            not_sp = re.sub('\n', '', i)
            not_sp = re.sub(' '*21, '', not_sp)
            command = re.findall("\S+ <[^<>]+>",not_sp)
            command_detail = {}
            for j in command:
                detail = re.match(r'(\S+) <(.*)>',j)
                key_tmp = re.sub(',','',detail.group(1))
                command_detail[key_tmp]=detail.group(2)
            dict_output[command_detail['Job']] = command_detail
    return dict_output

def read_bjobs_output(file_name):
    with open(file_name, 'r') as f:
        all_text = f.read()
        info = string.split(all_text, 'The output (if any) follows:')
        while '' in info:
            info.remove('')
        output = info[1]
        bjobs_inf = info[0]
        bjobs_inf = string.split(bjobs_inf,"\n")
    return bjobs_inf,output



import matplotlib

matplotlib.use('Agg')

import matplotlib.pyplot as plt
import numpy as np
from matplotlib.ticker import MultipleLocator, FormatStrFormatter
from matplotlib import cm
from lib.common.fileIO import tsv_file_dict_parse
import matplotlib.gridspec as gridspec


def add_high(data, num, strand):
    new_array = []
    for i in range(0, int(data.shape[0])):
        for j in range(0, num):
            if strand == "+":
                new_array.append(data[i,])
            else:
                new_array.append(list(reversed(data[i,])))
    return np.array(new_array)


file = '/lustre/home/xuyuxing/Work/Other/Pseudouracil/all_tran/printdata2.txt'

data_dir = tsv_file_dict_parse(file)

intersection_list = {"WT": ("S02_S01", "S06_S05", "S10_S26"),
                     "sT": ("S04_S03", "S08_S07", "S12_S11"),
                     "Wm": ("S18_S17", "S22_S30", "S32_S31"),
                     "sm": ("S14_S13", "S16_S15", "S20_S19")}

id = 'ATCG01210.1'
strand = "+"
site = 136479

a = list(range(site - 20, site + 21))

peak_data_WT = []
for peak_tab in intersection_list['WT']:
    peak_data_WT.append([float(data_dir[i][peak_tab]) for i in data_dir if int(data_dir[i]['site']) in a])

peak_data_sT = []
for peak_tab in intersection_list['sT']:
    peak_data_sT.append([float(data_dir[i][peak_tab]) for i in data_dir if int(data_dir[i]['site']) in a])

peak_data_WT = add_high(np.array(peak_data_WT), 10, strand)
peak_data_sT = add_high(np.array(peak_data_sT), 10, strand)

# plt.style.use('ggplot')

# Make plot with vertical (default) colorbar
# fig, (ax0, ax1) = plt.subplots(2, 1)

#plt.figure(figsize=(25, 10))

# gs = gridspec.GridSpec(17, 1, height_ratios=[200, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1])
gs = gridspec.GridSpec(1, 1, height_ratios=[1])

ax0 = plt.subplot(gs[0])
# plt.set_size_inches(18.5, 10.5)

# plt.subplots_adjust(wspace=0, hspace=-100)

cax0 = ax0.imshow(np.array(list(peak_data_WT) + list(peak_data_sT)), interpolation='nearest', cmap=cm.YlOrRd)
plt.colorbar(cax0, ax=ax0)
# cax0 = ax0.imshow(peak_data_WT, interpolation='nearest', cmap=cm.YlOrRd)
# cax0 = ax0.imshow(peak_data_WT)
ax0.spines['top'].set_visible(False)
ax0.spines['right'].set_visible(False)
ax0.spines['bottom'].set_visible(False)
ax0.spines['left'].set_visible(False)
ax0.get_xaxis().set_visible(False)
ax0.set_xlim(0, len(peak_data_WT[0]))
line2 = ax0.axhline(y=29, color='w', linewidth=6)

WT_coverage_p = []
WT_coverage_n = []
for peak_tab in intersection_list['WT']:
    CMC_p, CMC_n = peak_tab.split('_')
    WT_coverage_p.append([float(data_dir[i][CMC_p]) for i in data_dir if data_dir[i]['Gene_id'] == id])
    WT_coverage_n.append([float(data_dir[i][CMC_n]) for i in data_dir if data_dir[i]['Gene_id'] == id])

#xmajorLocator = MultipleLocator(200)

for ax_tmp in [ax0]:
    #ax_tmp.xaxis.set_major_locator(xmajorLocator)
    ax_tmp.yaxis.set_major_locator(plt.NullLocator())

# plt.axis('off')

# plt.grid(False)
plt.savefig('/lustre/home/xuyuxing/Work/Other/Pseudouracil/all_tran/' + id + '_' + str(site) + '.pdf', dpi=1000)
# fig.savefig('25S rRNA.pdf', dpi=1000)

plt.show()



# tools from book: ISLR Seventh printing
import numpy as np

y = [1, 2, 3, 4, 5]
fx = [2, 3, 4, 5, 6]


def mean_squared_error(y, fx):
    y = np.array(y)
    fx = np.array(fx)
    return np.average((y - fx) ** 2)




from pyfaidx import Fasta
from collections import OrderedDict
from Bio.Seq import Seq
from Bio.SeqRecord import SeqRecord
from Bio.SeqFeature import SeqFeature, FeatureLocation
from BCBio import GFF
from lib.common.genome.seq_base import reverse_complement, read_fasta_by_faidx, get_seq_index_ignore_gap
from lib.common.os import rmdir, cmd_run
from lib.common.util import printer_list
from lib.common.util import logging_init
from api.xuyuxing.genome.trf import trf_dat_to_dict
from lib.common.math.set import merge_same_element_set
from lib.common.math.interval import section, merge_intervals
from itertools import combinations
from api.common.genome.blast import outfmt5_read_big, blastn_running, outfmt5_read
from api.xuyuxing.genome.genblasta import GenBlastAJob
from lib.common.fileIO import tsv_file_dict_parse
import os


def repeat_unit_face(one_of_unit_face):
    #    one_of_unit_face = "CCCTAAA"
    double_unit = one_of_unit_face + one_of_unit_face
    unit_len = len(one_of_unit_face)

    all_face = []
    for i in range(unit_len):
        face_tmp = double_unit[i:i + unit_len]
        all_face.append(face_tmp)
        all_face.append(reverse_complement(face_tmp))

    all_face = sorted(list(set(all_face)))

    return all_face

def Telomere_main(args):

    fasta_file = args.fasta_file
    trf_dat = args.dat
    trf_path = args.trf_path
    log_file = args.log_file
    output_prefix = args.output_prefix

    """
    fasta_file = "/lustre/home/xuyuxing/Database/Other_genome/Ath/Tair10/TAIR10_Chr.all.fasta"
    trf_dat = "/lustre/home/xuyuxing/Database/Other_genome/Ath/Tair10/TAIR10_Chr.all.fasta.2.7.7.80.10.20.10.dat"
    log_file = "/lustre/home/xuyuxing/Database/Other_genome/Ath/Tair10/log"
    output_prefix =  "/lustre/home/xuyuxing/Database/Other_genome/Ath/Tair10/Ath_"
    """

    logger = logging_init("Telomere", log_file)

    logger.info("Find telomeres in an assembly")
    logger.info("Step1: get the data file from trf program")
    if trf_dat is None:
        logger.info("do not have a dat file, running trf now")
        cmd_string = "%s %s 2 7 7 80 10 20 10 -d -h" % (trf_path, fasta_file)
        a = cmd_run(cmd_string, silence=True, log_file=log_file)
        fasta_file_name = fasta_file.split("/")[-1]
        trf_dat = fasta_file_name + ".2.7.7.80.10.20.10.dat"
    else:
        logger.info("have a dat file already, pass step1")
    logger.info("Step1: finished")

    logger.info("Step2: parse trf dat file")
    trf_dict = trf_dat_to_dict(trf_dat)
    logger.info("Step2: finished")

    logger.info("Step3: find telomere unit")
    reference_dict = Fasta(fasta_file)
    telomere_dict = {}
    for chrome in reference_dict.keys():
        for trf_record in trf_dict[chrome]:
            start = min(int(trf_record["start"]), int(trf_record["end"]))
            end = max(int(trf_record["start"]), int(trf_record["end"]))
            if start <= 10 or len(reference_dict[chrome]) - end <= 10:
                uniq_rep_face = repeat_unit_face(trf_record["unit"])[0]
                if not uniq_rep_face in telomere_dict:
                    telomere_dict[uniq_rep_face] = []
                telomere_dict[uniq_rep_face].append(trf_record)
    logger.info("Step3: finished")

    logger.info("Step4: output")
    num = 0
    for i in sorted(telomere_dict, key=lambda x: len(telomere_dict[x]), reverse=True):
        with open(output_prefix + "telomere_" + str(num), "w") as f:
            for j in telomere_dict[i]:
                output_list = [i, j['chr'], j['start'], j['end'], j['unit'], j['period_size'], j['copy_num']]
                printer = printer_list(output_list)
                f.write(printer + "\n")
        num = num + 1
    logger.info("Step4: finished")


import os
import re
from collections import OrderedDict
from itertools import combinations
from config import trf_path
from api.xuyuxing.genome.trf import trf_dat_to_dict
from api.common.genome.blast import blastn_running, outfmt6_read_big
from lib.common.genome.seq_base import read_fasta_by_faidx
from lib.xuyuxing.base.base_function import merge_file
from lib.common.os import mkdir, rmdir, get_file_name, cmd_run, multiprocess_running
from lib.common.util import logging_init
from lib.common.math.set import merge_same_element_set
from lib.common.math.interval import section


def good_overlap(trf1, trf2):
    """
    function to find if two trf record is same loci but diff patten, used in step2 filter trf
    :param trf1:
    :param trf2:
    :return:
    """
    If_sect, delta = section(
        (trf1['start'], trf1['end']), (trf2['start'], trf2['end']))

    if If_sect:
        trf1_len = abs(trf1['start'] - trf1['end']) + 1
        trf2_len = abs(trf2['start'] - trf2['end']) + 1
        delta_len = abs(delta[1] - delta[0]) + 1
        if (delta_len / trf1_len) > 0.8 and (delta_len / trf2_len) > 0.8:
            return True
    return False


def remove_overlaped_trf(trf_list, trf_dict_ID):

    overlap_trfs = []
    for pairs in combinations(trf_list, 2):
        trf1 = trf_dict_ID[pairs[0]]
        trf2 = trf_dict_ID[pairs[1]]
        if good_overlap(trf1, trf2):
            overlap_trfs.append([pairs[0], pairs[1]])
    overlap_trfs = merge_same_element_set(overlap_trfs)
    for trf_group in overlap_trfs:
        for i in sorted(trf_group, key=lambda x: trf_dict_ID[x]['period_size'])[1:]:
            trf_list.remove(i)

    return trf_list


def CentromereSeed_main(args):
    mkdir(args.work_dir, True)
    log_file = args.work_dir + "/log"

    logger = logging_init("Centromere", log_file)

    logger.info("Find Centromere in an assembly")
    logger.info("Step1: get the data file from trf program")

    if args.trf_dat is None:
        logger.info("do not have a dat file, running trf now")

        trf_dir = args.work_dir + "/trf_dir"
        mkdir(trf_dir, True)
        split_trf_dir = trf_dir + "/split_dir"
        mkdir(split_trf_dir, True)

        record_dict = read_fasta_by_faidx(args.fasta_file)
        num = 0
        args_list = []
        for i in record_dict:
            record_tmp = record_dict[i]
            file_name = "seq%d.fa" % num
            with open(split_trf_dir + "/" + file_name, 'w') as f:
                f.write(">%s\n%s" % (record_tmp.seqname, record_tmp.seq))
            cmd_string = "%s %s 1 1 2 80 5 200 2000 -d -h" % (
                trf_path, file_name)
            print(cmd_string)
            args_list.append((cmd_string, split_trf_dir, 5, True, None))
            num += 1

        multiprocess_running(cmd_run, args_list,
                             args.threads, log_file=log_file)

        sub_file_list = []
        for i in os.listdir(split_trf_dir):
            if re.match(r'^.*\.dat$', i):
                sub_file_list.append("%s/%s" % (split_trf_dir, i))

        trf_dat = trf_dir + "/" + \
            get_file_name(args.fasta_file) + ".1.1.2.80.5.200.2000.dat"

        merge_file(sub_file_list, trf_dat)

        # rmdir(split_trf_dir)
    else:
        logger.info("have a dat file already, pass step1")
        trf_dat = args.trf_dat

    logger.info("Step1: finished")

    logger.info("Step2: parse trf dat file and filter same loci")

    trf_dict = trf_dat_to_dict(trf_dat)
    # data shift
    trf_dict_ID = {}
    for contig in trf_dict:
        num = 0
        for i in trf_dict[contig]:
            trf_element_ID = "%s_%d" % (contig, num)
            num = num + 1
            trf_dict_ID[trf_element_ID] = i

    trf_dict_ID_contig_hash = {}
    for trf_ID in trf_dict_ID:
        if trf_dict_ID[trf_ID]["chr"] not in trf_dict_ID_contig_hash:
            trf_dict_ID_contig_hash[trf_dict_ID[trf_ID]["chr"]] = []
        trf_dict_ID_contig_hash[trf_dict_ID[trf_ID]["chr"]].append(trf_ID)

    # filter: remove overlaped but bigger period size record
    filter_ID = {}
    args_list = []
    args_id_list = []
    for contig in trf_dict_ID_contig_hash:
        if len(trf_dict_ID_contig_hash[contig]) >= 2:
            trf_list = trf_dict_ID_contig_hash[contig]
            args_list.append((trf_list, trf_dict_ID))
            args_id_list.append(contig)
        else:
            filter_ID[contig] = trf_dict_ID_contig_hash[contig]

    mp_out = multiprocess_running(
        remove_overlaped_trf, args_list, 56, None, True, args_id_list)

    for i in mp_out:
        filter_ID[i] = mp_out[i]['output']

    # filter to small trf
    filter_dict_ID = {}
    for contig in filter_ID:
        for i in filter_ID[contig]:
            if trf_dict_ID[i]['period_size'] > 100:
                filter_dict_ID[i] = trf_dict_ID[i]

    logger.info("Step2: finished")

    logger.info("Step3: merge same unit")

    # blast
    trf_dir = args.work_dir + "/trf_dir"
    mkdir(trf_dir, True)

    trf_unit_fasta_file = trf_dir + "/trf_unit.fa"

    with open(trf_unit_fasta_file, 'w') as f:
        for i in filter_dict_ID:
            trf = filter_dict_ID[i]
            f.write(">%s\n%s\n" % (i, trf['unit']))

    trf_two_unit_fasta_file = trf_dir + "/trf_two_unit.fa"

    with open(trf_two_unit_fasta_file, 'w') as f:
        for i in filter_dict_ID:
            trf = filter_dict_ID[i]
            f.write(">%s\n%s%s\n" % (i, trf['unit'], trf['unit']))

    trf_unit_self_bls_file = trf_dir + "/trf_unit.bls"
    blastn_running(trf_unit_fasta_file, trf_two_unit_fasta_file,
                   trf_unit_self_bls_file, 1e-10, 56, realy_run=True, outfmt=6, task="blastn")

    # merge same unit trf

    trf_len_dir = {i: trf_dict_ID[i]['period_size'] for i in trf_dict_ID}

    one_way_dict = {}
    num = 0
    for record in outfmt6_read_big(trf_unit_self_bls_file):
        num += 1
        if num % 1000 == 0:
            print(num)
        one_way_dict[record.qID] = []
        for hit in record.hit:
            if hit.Hit_id == record.qID:
                continue

            top_hsp = hit.hsp[0]
            if top_hsp.Hsp_identical_ratio > 0.95 and top_hsp.Hsp_align_len / trf_len_dir[record.qID] > 0.95:
                one_way_dict[record.qID].append(hit.Hit_id)

    pair_list = []
    for i in one_way_dict:
        i_list = []
        for j in one_way_dict[i]:
            if j in one_way_dict and i in set(one_way_dict[j]):
                i_list.append(j)
        if len(i_list) > 0:
            i_list.append(i)
            pair_list.append(i_list)

    merged_list = merge_same_element_set(pair_list)

    merged_trf_dict = {}
    for i, j in enumerate(merged_list):
        merged_trf_dict[i] = j

    logger.info("Step3: finished")

    logger.info("Step4: find top candidate centromere unit")
    merged_trf_sum_dict = {}
    for m_id in merged_trf_dict:
        merged_trf_sum_dict[m_id] = sum([float(trf_dict_ID[trf_element_ID]['period_size']) * float(
            trf_dict_ID[trf_element_ID]['copy_num']) for trf_element_ID in merged_trf_dict[m_id]])

    # make huge report
    column_list = ['cluster_ID', 'cluster_sum_length', 'trf_ID', 'chr', 'start', 'end', 'period_size', 'copy_num',
                   'consensus_size', 'matches', 'indels', 'score', 'A_perc', 'C_perc', 'G_perc', 'T_perc', 'entropy', 'unit']

    with open(args.work_dir + "/trf_report.txt", 'w') as f:
        f.write("\t".join(column_list) + "\n")
        for c_id in sorted(merged_trf_sum_dict, key=lambda x: merged_trf_sum_dict[x], reverse=True):
            for trf_ID in sorted(merged_trf_dict[c_id], key=lambda x: len(trf_dict_ID[x]['sequence']), reverse=True):
                print_string = "\t".join(["Cluster_" + str(c_id), str(merged_trf_sum_dict[c_id]), trf_ID] + [str(trf_dict_ID[trf_ID][i]) for i in [
                                         'chr', 'start', 'end', 'period_size', 'copy_num', 'consensus_size', 'matches', 'indels', 'score', 'A_perc', 'C_perc', 'G_perc', 'T_perc', 'entropy', 'unit']])
                f.write(print_string + "\n")


if __name__ == "__main__":
    class abc():
        pass

    args = abc()

    args.trf_dat = None
    args.fasta_file = '/lustre/home/xuyuxing/Database/Lindenbergia/genome/hic_canu/contig_filter/nextpolish2_canu.rawchrom.fasta'
    args.work_dir = '/lustre/home/xuyuxing/Database/Lindenbergia/genome/hic_canu/contig_filter/Centromere'
    args.threads = 56



import sys

sys.path.append("/lustre/home/xuyuxing/python_project/Genome_work_tools/")


import re
import json
import networkx as nx
import uuid
import numpy as np
from mpl_toolkits.axisartist.axislines import SubplotZero
import matplotlib.pyplot as plt
from interlap import InterLap
import pandas as pd
from collections import Counter
from itertools import combinations
from lib.xuyuxing.math.set_operating import interval_length_sum
from lib.common.math.interval import merge_intervals, interval_minus_set, overlap_between_interval_set, \
    group_by_intervals_with_overlap_threshold
from api.xuyuxing.plot import gene_structure_plot, exon_box
import lib.common.sqlite_command as sc
from lib.common.genome.genome_feature2 import read_gff_file, get_gf_db_meta_dict, gf_info_retrieval, write_gff_file
from lib.common.genome.seq_base import read_fasta_by_faidx
from api.xuyuxing.genome.repeatmasker import repeatmasker_parser
from api.xuyuxing.genome.cd_hit import cd_hit_runing
from lib.common.util import printer_list
from lib.common.math.bin_and_window import split_sequence_to_bins
from lib.common.os import mkdir, multiprocess_running


def load_map_files(reference_genome_table, WPGmapper_dir):
    # read ref genome taxon_id
    ref_db = pd.read_excel(reference_genome_table)
    ref_taxon_id_list = []
    for index in ref_db.index:
        taxon_id = str(ref_db.loc[index]['id'])
        if not pd.isna(taxon_id):
            ref_taxon_id_list.append(taxon_id)

    # load map database
    ref_WPGmapper_dict = {}
    for taxon_id in ref_taxon_id_list:
        ref_WPGmapper_dict[taxon_id] = {}
        ref_WPGmapper_dict[taxon_id]['genblasta'] = WPGmapper_dir + \
            "/WPGmapper/" + taxon_id + "/genblasta.db"
        ref_WPGmapper_dict[taxon_id]['genewise'] = WPGmapper_dir + \
            "/WPGmapper/" + taxon_id + "/genewise.db"
        ref_WPGmapper_dict[taxon_id]['pep_db'] = WPGmapper_dir + \
            "/WPGmapper/" + taxon_id + "/genewise.pep.db"
        ref_WPGmapper_dict[taxon_id]['cds_db'] = WPGmapper_dir + \
            "/WPGmapper/" + taxon_id + "/genewise.cds.db"

    return ref_WPGmapper_dict


def min_cds_start(cds_range_list):
    return min([min(i) for i in cds_range_list])


def get_overlap_ratio(gf1,gf2,similarity_type):
    gf1_cds_interval = [(j.start, j.end) for j in gf1.sub_features[0].sub_features]
    gf2_cds_interval = [(j.start, j.end) for j in gf2.sub_features[0].sub_features]

    overlap_ratio, overlap_length, overlap = overlap_between_interval_set(gf1_cds_interval, gf2_cds_interval, similarity_type=similarity_type)
    
    return overlap_ratio


def higher_in_lower_group(lower_merged_group, higher, cds_interval_dict, similarity_type, threshold):
    grouped_flag = False

    for lower in lower_merged_group:
        jaccord_score = overlap_between_interval_set(
            cds_interval_dict[lower], cds_interval_dict[higher], similarity_type)[0]
        if jaccord_score >= threshold:
            grouped_flag = True
            break

    return grouped_flag


def merge_evidence_by_cds_intervals_identity(ev_id_list, cds_interval_dict, similarity_type, threshold):

    sorted_by_lower_bound = sorted(
        ev_id_list, key=lambda x: min_cds_start(cds_interval_dict[x]))
    merged = Counter()

    num = 0
    for higher in sorted_by_lower_bound:
        if len(merged) == 0:
            merged[num] = [higher]
            num += 1
        else:
            g_id_list = list(merged.keys())
            g_id_list.reverse()

            grouped_flag = False
            for g_id in g_id_list:
                grouped_flag = higher_in_lower_group(
                    merged[g_id], higher, cds_interval_dict, similarity_type, threshold)
                if grouped_flag == True:
                    merged[g_id].append(higher)
                    break

            if not grouped_flag:
                merged[num] = [higher]
                num += 1

    return merged


def extract_evidence(contig, strand, ref_WPGmapper_dict, min_cover, min_aa_len, min_identity, given_range=None):

    # get all useable evidence on the contig(strand)

    evidence_dict = {}

    for taxon_id in ref_WPGmapper_dict:
        # print(taxon_id)

        gws_db = ref_WPGmapper_dict[taxon_id]['genewise']
        meta_dict = get_gf_db_meta_dict(gws_db)

        # read mRNA info

        for table_name in meta_dict['gene'][0]:
            # print(table_name)
            if given_range is None:
                sql_cmd_string = 'SELECT * FROM %s WHERE contig_name = \"%s\" AND strand = \"%s\"' % (
                    table_name, contig, strand)
            else:
                sql_cmd_string = 'SELECT * FROM %s WHERE contig_name = \"%s\" AND strand = \"%s\" AND start <= %d AND end >= %d' % (
                    table_name, contig, strand, max(given_range), min(given_range))

            data_list = sc.sqlite_execute(sql_cmd_string, gws_db)
            for gf_db_info_tuple in data_list:
                id_tmp, gf_name, type_tmp, contig_name, start, end, strand, daughter, qualifiers = gf_db_info_tuple
                gf = gf_info_retrieval(gf_db_info_tuple, 0, 'A', gws_db)

                mRNA_gf = gf.sub_features[0]
                qualifiers = mRNA_gf.qualifiers

                # query coverage
                coverage = min((int(qualifiers['Target_End'][0]) - int(
                    qualifiers['Target_Start'][0]) + 1) / int(qualifiers['Target_Length'][0]), 1.00)

                # aln aa
                aln_aa_len = int(
                    qualifiers['Target_End'][0]) - int(qualifiers['Target_Start'][0]) + 1

                # identity
                identity = int(qualifiers['idt_len']) / \
                    int(qualifiers['aln_len'])

                # filter
                evidence_id = "%s.%s.%s" % (taxon_id, table_name, id_tmp)

                gf.evidence_indicator = {
                    "query_coverage":coverage,
                    "aln_aa_len":aln_aa_len,
                    "identity":identity,
                }

                if coverage >= min_cover and aln_aa_len >= min_aa_len and identity >= min_identity:
                    gf.map_quality_pass = True
                    gf.cl_cover_pass = None
                else:
                    gf.map_quality_pass = False
                    gf.cl_cover_pass = False
                    gf.evidence_indicator["evidence_range_cover"]=0.0

                evidence_dict[evidence_id] = gf

    return evidence_dict


def evidence_cluster(evidence_dict, similarity_type='jaccord_score', threshold=0.3):
    if len(evidence_dict) > 0:

        # CDS interval list
        cds_interval_dict = {}
        gene_interval_dict = {}
        for i in evidence_dict:
            if not evidence_dict[i].map_quality_pass:
                continue

            gene_interval_dict[i] = (
                evidence_dict[i].start, evidence_dict[i].end)
            cds_interval_dict[i] = [
                (j.start, j.end) for j in evidence_dict[i].sub_features[0].sub_features]

        big_group_dict = group_by_intervals_with_overlap_threshold(gene_interval_dict, 0)

        # # load evidences into interlap
        # evidence_interlap = InterLap()
        # evidence_interlap.update(
        #     [(gene_interval_dict[i][0], gene_interval_dict[i][1], i) for i in gene_interval_dict])

        # merge intervals into cluster
        ev_cluster_dict = {}
        num = 0
        for bg_id in big_group_dict:
            cds_interval_dict_in_group = {ev_id:cds_interval_dict[ev_id] for ev_id in big_group_dict[bg_id]['list']}

            ev_cluster_dict_in_group = merge_evidence_by_cds_intervals_identity(
                list(cds_interval_dict_in_group.keys()), cds_interval_dict_in_group, similarity_type, threshold)
            
            for i in ev_cluster_dict_in_group:
                num += 1
                ev_cluster_dict[num] = ev_cluster_dict_in_group[i]

        # ev_cluster_range_dict = {}
        # for cl_id in ev_cluster_dict:
        #     ev_cluster_range_dict[cl_id] = merge_intervals([gene_interval_dict[i] for i in ev_cluster_dict[cl_id]])[0]

        return ev_cluster_dict

    else:
        return Counter()


# def build_skip_interlap_dict(contig_list, target_gene_gff, target_repeatmasker):

#     target_gene_dict = read_gff_file(target_gene_gff)
#     target_repeat_dict = repeatmasker_parser(target_repeatmasker)

#     skip_interlap_dict = {contig_id: [] for contig_id in contig_list}
#     for repeat_type in target_repeat_dict:
#         for family_name in target_repeat_dict[repeat_type]:
#             for case in target_repeat_dict[repeat_type][family_name].case_list:
#                 contig = case.q_name
#                 start = case.q_start
#                 end = case.q_end

#                 if contig not in skip_interlap_dict:
#                     continue

#                 skip_interlap_dict[contig].append(
#                     (min(start, end), max(start, end)))

#     for gf_type in target_gene_dict:
#         for gene_id in target_gene_dict[gf_type]:
#             gf = target_gene_dict[gf_type][gene_id]
#             contig = gf.chr_id
#             start = gf.start
#             end = gf.end

#             if contig not in skip_interlap_dict:
#                 continue

#             skip_interlap_dict[contig].append(
#                 (min(start, end), max(start, end)))

#     for contig in skip_interlap_dict:
#         range_list = merge_intervals(skip_interlap_dict[contig])
#         skip_interlap_dict[contig] = InterLap()
#         if len(range_list) > 0:
#             skip_interlap_dict[contig].update(range_list)

#     return skip_interlap_dict

def read_skip_file(skip_file, contig_list):
    skip_range_dict = {i: InterLap() for i in contig_list}

    if skip_file is None:
        return skip_range_dict
    else:
        with open(skip_file, 'r') as f:
            for each_line in f:

                each_line = each_line.strip()
                chr_id, start, end = each_line.split()
                start = int(start)
                end = int(end)

                skip_range_dict[chr_id].add((min(start, end), max(start, end)))

        return skip_range_dict


def get_seq_from_db(seq_id, db_file):
    output_tuple = sc.sqlite_select(
        db_file, 'record', ['seqname_short', 'seqs'], 'seqname_short', [seq_id])
    return output_tuple[0][1]


def pseudo_judge(pseudo_event, evidence_gf_dict, event_range, ref_WPGmapper_dict):

    # cds interlap
    all_evidence_cds_interlap = InterLap()
    for ev_id in evidence_gf_dict:
        ggf = evidence_gf_dict[ev_id]
        if not ggf.cl_cover_pass:
            continue
        gf = ggf.sub_features[0]
        for sub_gf in gf.sub_features:
            all_evidence_cds_interlap.add(
                (sub_gf.start, sub_gf.end, (ev_id, sub_gf.id)))

    merged_pseudo_event = {}
    for site_range, ev_id, pseudo_type in pseudo_event:
        if pseudo_type not in merged_pseudo_event:
            merged_pseudo_event[pseudo_type] = InterLap()
        merged_pseudo_event[pseudo_type].add(
            (site_range[0]-int(event_range/2), site_range[1]+int(event_range/2), (site_range, ev_id)))

    output_pseudo_dict = {}
    all_support_ev_list = []
    for pseudo_type in merged_pseudo_event:
        output_pseudo_dict[pseudo_type] = {}
        for merge_range in merge_intervals([(i[0], i[1]) for i in merged_pseudo_event[pseudo_type]]):

            tmp_list = []
            support_ev_list = []
            for i in merged_pseudo_event[pseudo_type].find(merge_range):
                tmp_list.append(i[2][0][0])
                tmp_list.append(i[2][0][1])
                support_ev_list.append(i[2][1])

            merge_pseudo_event_range = (min(tmp_list), max(tmp_list))
            support_ev_list = tuple(list(set(support_ev_list)))
            all_support_ev_list.extend(support_ev_list)

            all_ev_list = tuple(list(
                set([i[2][0] for i in all_evidence_cds_interlap.find(merge_pseudo_event_range)])))

            if len(all_ev_list) == 0:
                support_ratio = 0
            else:
                support_ratio = len(support_ev_list) / len(all_ev_list)

            output_pseudo_dict[pseudo_type][merge_pseudo_event_range] = (
                support_ratio, support_ev_list, all_ev_list)

    # get representable gf

    if len(all_support_ev_list) > 0:
        taxon_list = [ev_id.split(".")[0] for ev_id in all_support_ev_list]

        seq_dict = {}
        cds_seq_dict = {}
        for ev_id in all_support_ev_list:
            taxon_id, table_name, id_tmp = ev_id.split(".")
            ggf = evidence_gf_dict[ev_id]
            gf = ggf.sub_features[0]
            gene_id = re.sub(r'.mRNA$', '', gf.id)

            seq_dict[ev_id] = get_seq_from_db(
                gene_id, ref_WPGmapper_dict[taxon_id]['pep_db'])
            cds_seq_dict[ev_id] = get_seq_from_db(gene_id,ref_WPGmapper_dict[taxon_id]['cds_db'])

        seq_cluster_dict = cd_hit_runing(seq_dict)

        if seq_cluster_dict is None:
            print("seq_cluster_dict is None， print seq_dict:")
            for i in seq_dict:
                print("%s\t%s\n" % (i, seq_dict[i]))

            raise ValueError("seq_cluster_dict is None")

        rep_ev_id = sorted(seq_cluster_dict, key=lambda x: len(
            seq_cluster_dict[x]), reverse=True)[0]

        rep_gf = evidence_gf_dict[rep_ev_id]
        rep_aa_seq = seq_dict[rep_ev_id]
        rep_cds_seq = cds_seq_dict[rep_ev_id]

    else:
        rep_gf = None
        rep_aa_seq = None
        rep_cds_seq = None

    return output_pseudo_dict, rep_gf, rep_aa_seq, rep_cds_seq
    # return output_pseudo_dict, rep_gf, rep_aa_seq


def filter_short_evidence(given_gf_dict, evidence_range_cover):
    total_ev_cds_range = []
    for ev_id in given_gf_dict:
        gf = given_gf_dict[ev_id]
        for sgf in gf.sub_features[0].sub_features:
            total_ev_cds_range.append((sgf.start, sgf.end))

    total_ev_cds_range = merge_intervals(total_ev_cds_range)
    print(total_ev_cds_range)

    # filter short evidence
    
    filtered_gf_dict = {}
    for ev_id in given_gf_dict:
        gf = given_gf_dict[ev_id]
        gf_cds_range_list = [(sgf.start, sgf.end)
                             for sgf in gf.sub_features[0].sub_features]
        print(gf_cds_range_list)
        overlap_ratio, overlap_length, overlap = overlap_between_interval_set(
            total_ev_cds_range, gf_cds_range_list)

        gf.evidence_indicator['evidence_range_cover'] = overlap_length / interval_length_sum(total_ev_cds_range)
        if gf.evidence_indicator['evidence_range_cover'] > evidence_range_cover:
            gf.cl_cover_pass = True
            print(gf, True)
        else:
            gf.cl_cover_pass = False
            print(gf, False)
        filtered_gf_dict[ev_id] = gf
    
    return filtered_gf_dict

    # for i,j in list(combinations(filtered_gf_dict,2)):
    #     print(i,j)
    #     gf = given_gf_dict[i]
    #     i_gf_cds_range_list = [(sgf.start, sgf.end)
    #                             for sgf in gf.sub_features[0].sub_features]
    #     gf = given_gf_dict[j]
    #     j_gf_cds_range_list = [(sgf.start, sgf.end)
    #                             for sgf in gf.sub_features[0].sub_features]

    #     print(overlap_between_interval_set(i_gf_cds_range_list,j_gf_cds_range_list,"jaccord_score"))



def pseudo_event_in_range(given_gf_dict, gene_fragment_ratio, event_range, ref_WPGmapper_dict, min_support_ratio, min_support_evidence_num, get_seq_flag=True):
    """
    cluster_range = (131484, 144903)
    """
    # get pseudo report
    # short pseudo
    # stop and shift pseudo

    pseudo_event = []

    for ev_id in given_gf_dict:
        gf = given_gf_dict[ev_id]
        if not gf.cl_cover_pass:
            continue
        mgf = gf.sub_features[0]

        # pseudo site
        pseudo_gf = False
        if "Note" in mgf.qualifiers and mgf.qualifiers["Note"][0] == "pseudogene":
            for sub_gf in mgf.sub_features:
                if "Note_stop_code" in sub_gf.qualifiers:
                    pseudo_gf = True
                    for site in sub_gf.qualifiers["Note_stop_code"]:
                        pseudo_event.append(
                            ((int(site), int(site)), ev_id, "Note_stop_code"))

                if "Note_frame_shift" in sub_gf.qualifiers:
                    pseudo_gf = True
                    for site in sub_gf.qualifiers["Note_frame_shift"]:
                        pseudo_event.append(
                            ((int(site), int(site)), ev_id, "Note_frame_shift"))

        # gene fragment
        coverage = min((int(mgf.qualifiers['Target_End'][0]) - int(
            mgf.qualifiers['Target_Start'][0]) + 1) / int(mgf.qualifiers['Target_Length'][0]), 1.00)

        if coverage <= gene_fragment_ratio:
            pseudo_event.append(((mgf.start, mgf.end), ev_id, "Gene_Fragment"))
            pseudo_gf = True

        if pseudo_gf == False:
            pseudo_event.append(((mgf.start, mgf.end), ev_id, "Missed_Gene"))

    pseudo_judge_dict, rep_gf, rep_aa_seq, rep_cds_seq = pseudo_judge(
        pseudo_event, given_gf_dict, event_range, ref_WPGmapper_dict)

    for pseudo_type in pseudo_judge_dict:
        for pseudo_site_range in pseudo_judge_dict[pseudo_type]:
            support_ratio, support_ev_list, all_ev_list = pseudo_judge_dict[
                pseudo_type][pseudo_site_range]
            if support_ratio >= min_support_ratio and len(support_ev_list) >= min_support_evidence_num:
                good_pseudo = True
            else:
                good_pseudo = False
            pseudo_judge_dict[pseudo_type][pseudo_site_range] = support_ratio, support_ev_list, all_ev_list, good_pseudo

    return pseudo_judge_dict, rep_gf, rep_aa_seq, rep_cds_seq


def pseudogene_main(args):
    mkdir(args.output_dir, True)

    contig_list = list(read_fasta_by_faidx(args.target_genome_fasta).keys())

    # read gff file
    # skip_interlap_dict = build_skip_interlap_dict(
    #     contig_list, args.target_gene_gff, args.target_repeatmasker)

    # skip_interlap_dict = read_skip_file(args.skip_range_file, contig_list)

    # load WPGmapper data
    ref_WPGmapper_dict = load_map_files(
        args.reference_genome_table, args.WPGmapper_dir)

    # parse a contig
    args_list = []
    for contig in contig_list:
        args_list.append((contig, ref_WPGmapper_dict, args))

    multiprocess_running(pseudogene_by_contig, args_list,
                         args.threads, silence=True)

# ok

def pseudogene_by_contig(contig, ref_WPGmapper_dict, args):

    pseudo_report = args.output_dir + "/%s.pseudo.txt" % contig
    evidence_report = args.output_dir + "/%s.evidence.txt" % contig
    pseudo_gene_gff = args.output_dir + "/%s.pseudo.gff" % contig
    pseudo_gene_seq = args.output_dir + "/%s.pseudo.faa" % contig
    pseudo_gene_cds_seq = args.output_dir + "/%s.pseudo.cds.fna" % contig

    p = open(pseudo_report, 'w')
    printer_string = "# cluster_id\tcontig\tstrand\tclst_start\tclst_end\ttotal_ev_num\tgood_map_ev_num\tgood_cover_ev_num\trepresent_ev_id\tpseudo_site_start\tpseudo_site_end\tpseudo_type\tsupport_ev_num\tall_ev_num\tsupport_ratio\tgood_pseudo\tsupport_ev_list\n"
    p.write(printer_string)

    e = open(evidence_report, 'w')
    printer_string = "# ev_id\tcontig\tstrand\tev_start\tev_end\tquery_coverage\taln_aa_len\tidentity\tgood_map\tev_range_cover\tgood_cover\n"
    e.write(printer_string)

    rep_gf_list = []
    rep_aa_seq_dict = {}
    rep_cds_seq_dict = {}

    for strand in ["+", "-"]:

        evidence_dict = extract_evidence(
            contig, strand, ref_WPGmapper_dict, args.min_cover, args.min_aa_len, args.min_identity)

        ev_cluster_ev_id_dict = evidence_cluster(
            evidence_dict, args.similarity_type, args.similarity_threshold)

        ev_cluster_dict = {}
        for cl_id in ev_cluster_ev_id_dict:
            print(cl_id)
            given_gf_dict = {ev_id:evidence_dict[ev_id] for ev_id in ev_cluster_ev_id_dict[cl_id]}
            given_gf_dict_add_filter = filter_short_evidence(given_gf_dict, args.evidence_range_cover)

            for i in given_gf_dict_add_filter:
                evidence_dict[i] = given_gf_dict_add_filter[i]

            pseudo_judge_dict, rep_gf, rep_aa_seq, rep_cds_seq = pseudo_event_in_range(given_gf_dict_add_filter, args.gene_fragment_ratio, args.event_range, ref_WPGmapper_dict, args.min_support_ratio, args.min_support_evidence_num, get_seq_flag=True)

            if not rep_gf is None:
                rep_gf_list.append(rep_gf)
                rep_aa_seq_dict[rep_gf.id] = rep_aa_seq
                rep_cds_seq_dict[rep_gf.id] = rep_cds_seq

            cl_name = contig+"_"+strand+"_"+str(cl_id)
            ev_cluster_dict[cl_name] = (pseudo_judge_dict, rep_gf, rep_aa_seq, rep_cds_seq)


        # make report
        used_ev_gf = []
        for ev_id in evidence_dict:
            ev_gf = evidence_dict[ev_id]

            printer_string = printer_list([ev_id, contig, strand, ev_gf.start, ev_gf.end, ev_gf.evidence_indicator["query_coverage"], ev_gf.evidence_indicator["aln_aa_len"],
                                           ev_gf.evidence_indicator["identity"], ev_gf.map_quality_pass, ev_gf.evidence_indicator["evidence_range_cover"], ev_gf.cl_cover_pass])

            if ev_gf.cl_cover_pass:
                used_ev_gf.append(ev_gf)

            print(printer_string)

        write_gff_file(used_ev_gf, "used_ev.gff3")

        pseudo_num = 0
        good_rep_gf = []
        best_ev_id = []
        for cl_name in ev_cluster_dict:
            pseudo_judge_dict, rep_gf, rep_aa_seq, rep_cds_seq = ev_cluster_dict[cl_name]
            
            pseudo_true_flag = False
            for pseudo_type in pseudo_judge_dict:
                for site_range in pseudo_judge_dict[pseudo_type]:
                    if pseudo_judge_dict[pseudo_type][site_range][3]:
                        pseudo_true_flag = True
                        best_ev_id.extend(list(pseudo_judge_dict[pseudo_type][site_range][2]))
            
            if pseudo_true_flag:
                pseudo_num += 1
                good_rep_gf.append(rep_gf)
        
        best_ev_id = list(set(best_ev_id))
        best_ev_gf_list = [evidence_dict[i] for i in best_ev_id]

        write_gff_file(good_rep_gf, "rep.gff3")
        write_gff_file(best_ev_gf_list, "good_ev.gff3")
        







    write_gff_file(rep_gf_list, pseudo_gene_gff, "Pseudo")

    with open(pseudo_gene_seq, 'w') as f:
        for i in rep_aa_seq_dict:
            f.write(">%s\n%s\n" % (i, rep_aa_seq_dict[i]))

    with open(pseudo_gene_cds_seq, 'w') as f:
        for i in rep_cds_seq_dict:
            f.write(">%s\n%s\n" % (i, rep_cds_seq_dict[i]))

# used now

def get_ev_weight(idt_len, query_length):
    if query_length > 200:
        length_score = 5
    elif query_length > 150:
        length_score = 4
    elif query_length > 100:
        length_score = 3
    elif query_length > 50:
        length_score = 2
    
    # return (idt_len/aln_len) * (aln_len/query_length) * length_score
    return (idt_len/query_length) * length_score


def make_cds_vector(evidence_dict_range, range_now):
    cds_weight_interlap = InterLap()

    coordinate_list = []
    for ev_id in evidence_dict_range:
        ev_gf = evidence_dict_range[ev_id]
        mRNA = ev_gf.sub_features[0]
        idt_len = mRNA.qualifiers['idt_len']
        query_length = int(mRNA.qualifiers['Target_Length'][0])
        ev_weight_value = get_ev_weight(idt_len, query_length)
        
        for cds in mRNA.sub_features:
            cds_weight_interlap.add((cds.start,cds.end,ev_weight_value))
            coordinate_list.append(cds.start)
            coordinate_list.append(cds.end)

    coordinate_list = sorted(list(set(coordinate_list)))

    cds_vector_interlap = InterLap()
    getted_range = {}
    for i in range(0,len(coordinate_list)-1):
        start = coordinate_list[i]
        end = coordinate_list[i+1]

        if (start, start) not in getted_range:
            w_s_total = sum([w for s,e,w in cds_weight_interlap.find((start, start))])
            getted_range[(start, start)] = w_s_total
            cds_vector_interlap.add((start,start,w_s_total))

        if (end, end) not in getted_range:
            w_e_total = sum([w for s,e,w in cds_weight_interlap.find((end, end))])
            getted_range[(end, end)] = w_e_total
            cds_vector_interlap.add((end,end,w_e_total))
        
        if end - start > 2:
            interval = (start + 1, end - 1)
            if interval not in getted_range:
                w_i_total = sum([w for s,e,w in cds_weight_interlap.find(interval)])
                getted_range[interval] = w_i_total
                cds_vector_interlap.add((interval[0],interval[1],w_i_total))
    
    return cds_vector_interlap

def get_vector_score(vector_interlap, start, end):

    w = [w for s,e,w in vector_interlap.find((start, end))]
    if len(w) > 1:
        raise ValueError("cds error: %d, %d" % (start, end))
    elif len(w) == 0:
        w = 0.0
    else:
        w = w[0]
    
    return w


def get_ev_gf_score(gf, cds_vector_interlap):

    mRNA = gf.sub_features[0]
    idt_len = mRNA.qualifiers['idt_len']
    query_length = int(mRNA.qualifiers['Target_Length'][0])
    ev_weight_value = get_ev_weight(idt_len, query_length)

    sum_w = 0
    length = 0
    for cds in mRNA.sub_features:
        for i in range(cds.start,cds.end+1):
            length += 1
            sum_w += get_vector_score(cds_vector_interlap, i, i)
    
    gf_score = (sum_w + ev_weight_value * length)/length

    return gf_score


def mark_ev_score_for_gf_dict(input_gf_dict, cds_vector_interlap):
    output_gf_dict = {}
    for i in input_gf_dict:
        gf = input_gf_dict[i]
        gf_score = get_ev_gf_score(gf, cds_vector_interlap)
        gf.gf_score = gf_score
        output_gf_dict[i] = gf
    return output_gf_dict

def mark_exclusivity_for_gf_dict(input_gf_dict):
    evidence_cluster_range = evidence_cluster(input_gf_dict, similarity_type='shorter_overlap_coverage', threshold=0.1)
    output_gf_dict = {}

    for cl_id in evidence_cluster_range:
        ev_id_list = evidence_cluster_range[cl_id]
        if len(ev_id_list) == 1:
            gf = input_gf_dict[ev_id_list[0]]
            gf.exclusivity = [ev_id_list[0]]
            output_gf_dict[ev_id_list[0]] = gf
        else:
            score_pair_dict = {}
            for i,j in combinations(ev_id_list, 2):
                score_pair_dict[(i,j)] = get_overlap_ratio(input_gf_dict[i],input_gf_dict[j],'shorter_overlap_coverage')

            for ev_id_i in evidence_cluster_range[cl_id]:
                gf = input_gf_dict[ev_id_i]
                exclusivity_list = []
                for ev_id_j in evidence_cluster_range[cl_id]:
                    if ev_id_i == ev_id_j:
                        continue
                    if (ev_id_i, ev_id_j) in score_pair_dict:
                        overlap_ratio = score_pair_dict[(ev_id_i, ev_id_j)]
                    else:
                        overlap_ratio = score_pair_dict[(ev_id_j, ev_id_i)]

                    if overlap_ratio > 0.5:
                        exclusivity_list.append(ev_id_j)
                gf.exclusivity = exclusivity_list
                gf.exclusivity.append(ev_id_i)
                output_gf_dict[ev_id_i] = gf
    
    return output_gf_dict
    
def load_a_node(graph, give_node_id, remaining_selectable_ev_list, evidence_dict_range):
    # print(give_node_id)
    # print(remaining_selectable_ev_list)
    uuid_id, seq_id = give_node_id

    for i in remaining_selectable_ev_list:
        uuid_id = uuid.uuid1().hex
        node_id = (uuid_id, i)

        graph.add_edge(give_node_id, node_id, weight=-evidence_dict_range[i].gf_score)

        remaining_selectable_ev_list_for_i = list(set(remaining_selectable_ev_list) - set(evidence_dict_range[i].exclusivity))
        if len(remaining_selectable_ev_list_for_i) == 0:
            graph.add_edge(node_id, 'END', weight=-1)
            # print(node_id)
        else:
            load_a_node(graph, node_id, remaining_selectable_ev_list_for_i, evidence_dict_range)


def best_select(evidence_dict_range):

    cds_vector_interlap = make_cds_vector(evidence_dict_range)
    evidence_dict_range = mark_ev_score_for_gf_dict(evidence_dict_range, cds_vector_interlap)
    evidence_dict_range = mark_exclusivity_for_gf_dict(evidence_dict_range)

    for i in evidence_dict_range:
        gf = evidence_dict_range[i]
        print(gf.id, gf.exclusivity)


def use_graph_to_get_rep(evidence_dict_range):
    
    G = nx.DiGraph()
    uuid_id = uuid.uuid1().hex
    start = (uuid_id, 'START')

    load_a_node(G, start, list(evidence_dict_range.keys()), evidence_dict_range)
    # paths = nx.johnson(G, weight="weight")
    # paths[start]["END"]

    return nx.shortest_path(G,start,"END",weight='weight',method="bellman-ford")

class abc():
    pass

evidence_dict_range2 = {}

gf1 = abc()
gf1.gf_score = 1000
gf1.exclusivity = ['gf1', 'gf2', 'gf3']
evidence_dict_range2['gf1'] = gf1

gf2 = abc()
gf2.gf_score = 600
gf2.exclusivity = ['gf1', 'gf2']
evidence_dict_range2['gf2'] = gf2

gf3 = abc()
gf3.gf_score = 600
gf3.exclusivity = ['gf1', 'gf3']
evidence_dict_range2['gf3'] = gf3

use_graph_to_get_rep(evidence_dict_range2)

G = nx.DiGraph()
G.add_edge(1, 2, weight=-1)
G.add_edge(1, 5, weight=-1)
G.add_edge(2, 3, weight=-1)
G.add_edge(2, 4, weight=-1)
G.add_edge(5, 6, weight=-1)
G.add_edge(5, 8, weight=-1)
G.add_edge(6, 7, weight=-1)
G.add_edge(3, 9, weight=-1)
G.add_edge(4, 9, weight=19)
G.add_edge(7, 9, weight=19)
G.add_edge(8, 9, weight=1)

nx.shortest_path(G,1,9,weight='weight')



# other

target_genome_dict = read_fasta_by_faidx(args.target_genome_fasta)
target_genome_dict = {
    i: str(target_genome_dict[i].seq) for i in target_genome_dict}


def test_frame(cds_start,cds_end,frame_shift):

    # cds_start = 8343856
    # cds_end = 8343952
    # frame_shift = 8343901

    seq_str = target_genome_dict[contig][cds_start - 1: cds_end].upper()
    print(seq_str)
    print(cds_judgment(seq_str, False))

    seq_str = target_genome_dict[contig][cds_start - 1: frame_shift-1].upper()
    print(seq_str)
    print(cds_judgment(seq_str, False))

    seq_str = target_genome_dict[contig][frame_shift + 1 - 1: cds_end].upper()
    print(seq_str)
    print(cds_judgment(seq_str))



def try_evm_annotation(contig, strand, ref_WPGmapper_dict):
    range_now = (8300000,8450000)
    evidence_dict = extract_evidence(contig, strand, ref_WPGmapper_dict, args.min_cover, args.min_aa_len, args.min_identity, (8300000,8450000))


    for ev_id in evidence_dict:
        ev_gf = evidence_dict[ev_id]
        
        aln_len = ev_gf.sub_features[0].qualifiers['aln_len']
        idt_len = ev_gf.sub_features[0].qualifiers['idt_len']
        query_length = int(ev_gf.sub_features[0].qualifiers['Target_Length'][0])
        ev_weight_value = get_ev_weight(idt_len, query_length)
        ev_name = ev_gf.sub_features[0].qualifiers['Parent'][0]

        ev_gf.ev_weight_value = ev_weight_value

    for ev_id in evidence_dict:
        ev_gf = evidence_dict[ev_id]
        ev_mRNA = ev_gf.sub_features[0]

        break_all = False

        for cds in ev_mRNA.sub_features:
            frame_site = []
            stop_site =[]
            if 'Note_frame_shift' in cds.qualifiers:
                frame_site = [int(i) for i in cds.qualifiers['Note_frame_shift']]
            if 'Note_stop_code' in cds.qualifiers:
                stop_site = [int(i) for i in cds.qualifiers['Note_stop_code']]
            
            cds_phase = int(cds.qualifiers['phase'][0])

            print(cds.start,cds.end,cds.strand,cds_phase,frame_site,stop_site)
    
            if frame_site != []:
                break_all = True
                break
        
        if break_all:
            break


def make_cds_vector(evidence_dict_range, range_now):
    cds_weight_interlap = InterLap()
    coordinate_list = [range_now[0],range_now[1]]

    for ev_id in evidence_dict_range:
        ev_gf = evidence_dict_range[ev_id]
        mRNA = ev_gf.sub_features[0]
        idt_len = mRNA.qualifiers['idt_len']
        query_length = int(mRNA.qualifiers['Target_Length'][0])
        ev_weight_value = get_ev_weight(idt_len, query_length)
        
        for cds in mRNA.sub_features:
            cds_weight_interlap.add((cds.start,cds.end,ev_weight_value))
            coordinate_list.append(cds.start)
            coordinate_list.append(cds.end)

    coordinate_list = sorted(list(set(coordinate_list)))

    cds_vector_interlap = InterLap()
    getted_range = {}
    for i in range(0,len(coordinate_list)-1):
        start = coordinate_list[i]
        end = coordinate_list[i+1]

        if (start, start) not in getted_range:
            w_s_total = sum([w for s,e,w in cds_weight_interlap.find((start, start))])
            getted_range[(start, start)] = w_s_total
            cds_vector_interlap.add((start,start,w_s_total))

        if (end, end) not in getted_range:
            w_e_total = sum([w for s,e,w in cds_weight_interlap.find((end, end))])
            getted_range[(end, end)] = w_e_total
            cds_vector_interlap.add((end,end,w_e_total))
        
        if end - start > 2:
            interval = (start + 1, end - 1)
            if interval not in getted_range:
                w_i_total = sum([w for s,e,w in cds_weight_interlap.find(interval)])
                getted_range[interval] = w_i_total
                cds_vector_interlap.add((interval[0],interval[1],w_i_total))
    
    return cds_vector_interlap

def get_vector_score(vector_interlap, start, end):

    w = [w for s,e,w in vector_interlap.find((start, end))]
    if len(w) > 1:
        raise ValueError("cds error: %d, %d" % (start, end))
    elif len(w) == 0:
        w = 0.0
    else:
        w = w[0]
    
    return w


def get_ev_gf_score(gf, cds_vector_interlap):

    mRNA = gf.sub_features[0]
    idt_len = mRNA.qualifiers['idt_len']
    query_length = int(mRNA.qualifiers['Target_Length'][0])
    ev_weight_value = get_ev_weight(idt_len, query_length)

    sum_w = 0
    length = 0
    for cds in mRNA.sub_features:
        for i in range(cds.start,cds.end+1):
            length += 1
            sum_w += get_vector_score(cds_vector_interlap, i, i)
    
    gf_score = (sum_w + ev_weight_value * length)/length

    return gf_score


for i in evidence_dict_range:
    print(evidence_dict_range[i].id, get_ev_gf_score(evidence_dict_range[i], cds_vector_interlap))


def make_start_cds_vector(evidence_dict_range, range_now):
    cds_weight_interlap = InterLap()
    coordinate_list = [range_now[0],range_now[1]]

    for ev_id in evidence_dict_range:
        ev_gf = evidence_dict_range[ev_id]
        mRNA = ev_gf.sub_features[0]
        idt_len = mRNA.qualifiers['idt_len']
        query_length = int(mRNA.qualifiers['Target_Length'][0])
        ev_weight_value = get_ev_weight(idt_len, query_length)
        
        for cds in mRNA.sub_features:
            cds_weight_interlap.add((cds.start,cds.end,ev_weight_value))
            coordinate_list.append(cds.start)
            coordinate_list.append(cds.end)

    coordinate_list = sorted(list(set(coordinate_list)))

    cds_vector_interlap = InterLap()
    getted_range = {}
    for i in range(0,len(coordinate_list)-1):
        start = coordinate_list[i]
        end = coordinate_list[i+1]

        if (start, start) not in getted_range:
            w_s_total = sum([w for s,e,w in cds_weight_interlap.find((start, start))])
            getted_range[(start, start)] = w_s_total
            cds_vector_interlap.add((start,start,w_s_total))

        if (end, end) not in getted_range:
            w_e_total = sum([w for s,e,w in cds_weight_interlap.find((end, end))])
            getted_range[(end, end)] = w_e_total
            cds_vector_interlap.add((end,end,w_e_total))
        
        if end - start > 2:
            interval = (start + 1, end - 1)
            if interval not in getted_range:
                w_i_total = sum([w for s,e,w in cds_weight_interlap.find(interval)])
                getted_range[interval] = w_i_total
                cds_vector_interlap.add((interval[0],interval[1],w_i_total))
    
    return cds_vector_interlap

def make_intron_vector(evidence_dict_range, range_now):
    intron_weight_interlap = InterLap()
    coordinate_list = [range_now[0],range_now[1]]

    for ev_id in evidence_dict_range:
        ev_gf = evidence_dict_range[ev_id]
        mRNA = ev_gf.sub_features[0]
        idt_len = mRNA.qualifiers['idt_len']
        query_length = int(mRNA.qualifiers['Target_Length'][0])
        ev_weight_value = get_ev_weight(idt_len, query_length)
        
        # get intron
        cds_coord = []
        for cds in mRNA.sub_features:
            cds_coord.append((cds.start,cds.end))
        
        cds_min  = min([min(i) for i in cds_coord])
        cds_max  = max([max(i) for i in cds_coord])

        intron_coord = interval_minus_set((cds_min,cds_max), cds_coord)

        for intron_start, intron_end in intron_coord:
            intron_weight_interlap.add((intron_start,intron_end,ev_weight_value))
            coordinate_list.append(intron_start)
            coordinate_list.append(intron_end)

    coordinate_list = sorted(list(set(coordinate_list)))

    intron_vector_interlap = InterLap()
    getted_range = {}
    for i in range(0,len(coordinate_list)-1):
        start = coordinate_list[i]
        end = coordinate_list[i+1]

        if (start, start) not in getted_range:
            w_s_total = sum([w for s,e,w in intron_weight_interlap.find((start, start))])
            getted_range[(start, start)] = w_s_total
            intron_vector_interlap.add((start,start,w_s_total))

        if (end, end) not in getted_range:
            w_e_total = sum([w for s,e,w in intron_weight_interlap.find((end, end))])
            getted_range[(end, end)] = w_e_total
            intron_vector_interlap.add((end,end,w_e_total))
        
        if end - start > 2:
            interval = (start + 1, end - 1)
            if interval not in getted_range:
                w_i_total = sum([w for s,e,w in intron_weight_interlap.find(interval)])
                getted_range[interval] = w_i_total
                intron_vector_interlap.add((interval[0],interval[1],w_i_total))
    
    return intron_vector_interlap






def make_intergenic_vector(cds_vector_interlap, intron_vector_interlap):
    # cds_intron_sum 
    coordinate_list = []

    for s,e,w in cds_vector_interlap:
        coordinate_list.append(s)
        coordinate_list.append(e)

    for s,e,w in intron_vector_interlap:
        coordinate_list.append(s)
        coordinate_list.append(e)    

    coordinate_list = sorted(list(set(coordinate_list)))

    intergenic_vector_plus_interlap = InterLap()
    getted_range = {}
    max_cds_intron = 0
    for i in range(0,len(coordinate_list)-1):
        start = coordinate_list[i]
        end = coordinate_list[i+1]

        if (start, start) not in getted_range:
            cds_w = get_vector_score(cds_vector_interlap, start, start)
            intron_w = get_vector_score(intron_vector_interlap, start, start)
            intergenic_vector_plus_interlap.add((start,start,cds_w+intron_w))
            getted_range[(start, start)] = cds_w+intron_w
            if cds_w+intron_w > max_cds_intron:
                max_cds_intron = cds_w+intron_w
        
        if (end, end) not in getted_range:
            cds_w = get_vector_score(cds_vector_interlap, end, end)
            intron_w = get_vector_score(intron_vector_interlap, end, end)
            intergenic_vector_plus_interlap.add((end,end,cds_w+intron_w))
            getted_range[(end, end)] = cds_w+intron_w
            if cds_w+intron_w > max_cds_intron:
                max_cds_intron = cds_w+intron_w
            
        if end - start > 2:
            interval = (start + 1, end - 1)
            if interval not in getted_range:
                cds_w = get_vector_score(cds_vector_interlap, interval[0], interval[1])
                intron_w = get_vector_score(intron_vector_interlap, interval[0], interval[1])
                intergenic_vector_plus_interlap.add((interval[0], interval[1],cds_w+intron_w))
                getted_range[interval] = cds_w+intron_w
                if cds_w+intron_w > max_cds_intron:
                    max_cds_intron = cds_w+intron_w

    intergenic_vector_interlap = InterLap()
    for s,e,w in intergenic_vector_plus_interlap:
        intergenic_vector_interlap.add((s, e, max_cds_intron - w))

    return intergenic_vector_interlap
    




def make_vector(evidence_dict_range, range_now):
    # cds vector
    cds_vector_interlap = make_cds_vector(evidence_dict_range, range_now)
    intron_vector_interlap = make_intron_vector(evidence_dict_range, range_now)
    intergenic_vector_interlap = make_intergenic_vector(cds_vector_interlap, intron_vector_interlap)




    # plot

    fig = plt.figure(figsize=(20, 30))

    ax1 = plt.subplot2grid((3, 1), (0, 0))
    ax2 = plt.subplot2grid((3, 1), (1, 0), sharex=ax1)
    ax3 = plt.subplot2grid((3, 1), (2, 0), sharex=ax1)

    ax1.plot(range(range_now[0],range_now[1]), [get_vector_score(cds_vector_interlap, i, i) for i in range(range_now[0],range_now[1])])
    ax2.plot(range(range_now[0],range_now[1]), [get_vector_score(intron_vector_interlap, i, i) for i in range(range_now[0],range_now[1])])
    ax3.plot(range(range_now[0],range_now[1]), [get_vector_score(intergenic_vector_interlap, i, i) for i in range(range_now[0],range_now[1])])

    plt.show()




def pseudogene_plot(contig, start, end, evidence_gf_dict):
    fig = plt.figure(figsize=(20, 10))
    # fig = plt.figure()
    # fig.subplots_adjust(right=2)
    ax = SubplotZero(fig, 1, 1, 1)
    fig.add_subplot(ax)

    # ax limit

    expand_length = int((end - start) * 0.3)

    ax.set_xlim(start - expand_length,
                end + expand_length)
    ax.set_ylim((-len(evidence_gf_dict) - 3), 1)
    ax.set_title('%s:%d-%d' % (contig, start, end))

    # plot
    num = 0
    for id_tmp in evidence_gf_dict:
        gf = evidence_gf_dict[id_tmp]
        num += 1
        gene_structure_plot(ax, gf, plot_feature_types=[
                            'CDS'], width=0.5, horizon=-num)

        # pseudo site
        if "Note" in gf.qualifiers and gf.qualifiers["Note"][0] == "pseudogene":
            for sub_gf in gf.sub_features:
                if "Note_stop_code" in sub_gf.qualifiers:
                    for site in sub_gf.qualifiers["Note_stop_code"]:
                        exon_box(ax, int(site), int(site),
                                 edgecolor='r', width=0.5, horizon=-num)
                if "Note_frame_shift" in sub_gf.qualifiers:
                    for site in sub_gf.qualifiers["Note_frame_shift"]:
                        exon_box(ax, int(site), int(site),
                                 edgecolor='r', width=0.5, horizon=-num)

    # axis set
    for direction in ["left", "right", "bottom", "top"]:
        # hides borders
        ax.axis[direction].set_visible(False)

    for direction in ["xzero"]:
        # adds arrows at the ends of each axis
        ax.axis[direction].set_axisline_style("-|>")

        # adds X and Y-axis from the origin
        ax.axis[direction].set_visible(True)

    ax.get_yaxis().set_visible(False)
    plt.show()


if __name__ == "__main__":
    class abc():
        pass

    args = abc()

    args.WPGmapper_dir = "/lustre/home/xuyuxing/Work/Gel/Gene_Loss/plant/pseudogene/PC/T267555N0C000/T267555N0C000.WPGmaper.new.skip_all"
    args.reference_genome_table = "/lustre/home/xuyuxing/Work/Gel/Gene_Loss/plant/pseudogene/PC/T267555N0C000/ref.xlsx"
    args.target_genome_fasta = "/lustre/home/xuyuxing/Work/Gel/Gene_Loss/plant/pseudogene/PC/T267555N0C000/T267555N0C000.WPGmaper.new.skip_all/T267555N0C000.genome.fasta"
    args.OG_tsv_file = "/lustre/home/xuyuxing/Work/Gel/WPGmapper/Gel_ortho/protein/OrthoFinder/Results_Oct06/Orthogroups/Orthogroups.tsv"
    args.output_dir = '/lustre/home/xuyuxing/Work/Gel/Gene_Loss/plant/pseudogene/PC/T267555N0C000/T267555N0C000.WPGmaper.new.skip_all.pseudo.test'

    # A valid evidence alignment region needs to account for at least min_cover of the reference gene
    args.min_cover = 0.5

    # A valid evidence alignment region needs at least min_aa_len in reference gene
    args.min_aa_len = 50

    # A valid evidence alignment region needs at least min_identity in genewise indentity
    args.min_identity = 0.2

    # A valid evidence alignment region needs at least min_identity in genewise indentity
    args.min_score = 50

    args.min_OG_support = 2

    args.min_av_supp_score = 200

    args.min_cluster_supp = 2

    # # Ignore if a evidence that the coverage in the discussion range is less than args.evidence_range_cover
    # args.evidence_range_cover = 0.5

    # # The area of evidence alignment that accounts for less than 50% of the reference gene is considered to be a gene fragment
    # args.gene_fragment_ratio = 0.5

    # # Events within event_range bases will be treated as the same event
    # args.event_range = 20

    # # Each event requires at least min_support_ratio evidence to support
    # args.min_support_ratio = 0.6

    # # Each event requires at least min_support_evidence_num evidence to support
    # args.min_support_evidence_num = 3

    # args.similarity_type = 'jaccord_score'
    
    # args.similarity_threshold = 0.6

    args.threads = 20

    # whole genome pseudogene calling
    pseudogene_main(args)











